/*****************************************************************************
 * cast.cpp        Illustration of the cast() function
 *
 * $Id: cast.cpp,v 1.1.1.1 2003/01/16 19:20:23 mf Exp $
 *
 * $Log: cast.cpp,v $
 * Revision 1.1.1.1  2003/01/16 19:20:23  mf
 * imported
 *
 * Revision 1.4  2002/07/02 19:48:44  jcumming
 * Added check for use of new-style expression templates.  I changed the syntax
 * for casting the type of a blitz Array to make it simpler.  Instead of giving
 * a second argument that is an instance of the new element type, just provide
 * the new element type as a template parameter.
 *
 * Revision 1.3  2001/01/26 18:30:50  tveldhui
 * More source code reorganization to reduce compile times.
 *
 * Revision 1.2  2001/01/24 23:41:53  tveldhui
 * Widespread changes to reduce compile time.  For backwards
 * compatibility, #include <blitz/array.h> enables BZ_GANG_INCLUDE
 * mode which includes all array and vector functionality (about
 * 120000 lines of code).  #include <blitz/Array.h> includes
 * a minimal subset of Array funcitonality; other features must
 * be included explicitly.
 *
 * Revision 1.1.1.1  2000/06/19 12:26:17  tveldhui
 * Imported sources
 *
 * Revision 1.1  1997/07/16 19:38:23  tveldhui
 * Update: Alpha release 0.2 (Arrays)
 *
 *****************************************************************************
 */

#include <blitz/array.h>

#ifdef BZ_NAMESPACES
using namespace blitz;
#endif

int main()
{
    Array<int,1> A(4), B(4);
    Array<float,1> C(4);

    A = 1, 2, 3, 5;
    B = 2, 2, 2, 7;

    C = A / B;
    cout << C << endl;

#ifdef BZ_NEW_EXPRESSION_TEMPLATES
    C = A / cast<float>(B);
#else
    C = A / cast(B, float());
#endif
    cout << C << endl;

    return 0;
}

