// simple linear algebra extensions

/*
J.M.P. Franosch (Jan-Moritz.Franosch@physik.tu-muenchen.de)

This file is part of the GNUSSL software package.  This package is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.  This software is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this distribution; if not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _linalg_ext_h_
#define _linalg_ext_h_


#include "mVector.h"
#include "utilities.h"




template<class T, class S1, class S2>
inline T scalar_product(const mVector<T,S1>& a,const mVector<T,S2>& b) {
#ifdef GNUSSL_CHECK
  if (a.size() != b.size()) {
    gnussl_error("scalar_product","vector sizes unequal");
  }
#endif
  T s=0;
  for (size_t i=1; i<=a.size(); i++) {
    s+=a(i)*b(i);
  }
  return s;
}


template<class T, class S1, class S2>
inline T euclidean_distance(const mVector<T,S1>& a,const mVector<T,S2>& b) {
#ifdef GNUSSL_CHECK
  if (a.size() != b.size()) {
    gnussl_error("distance","vector sizes unequal");
  }
#endif
  return euclidean_length(b-a);
}


template<class T, class S1>
inline T euclidean_length(const mVector<T,S1>& a) {
  return sqrt(norm(a));
}


template<class T, class S>
inline T trace(const mMatrix<T,S>& M) {
  T tr=0;
  for (size_t i=1; i<=min(M.rsize(),M.csize()); i++) {
    tr+=M(i,i);
  }
  return tr;
}




#endif // _linalg_ext_h_
