/*
J.M.P. Franosch (Jan-Moritz.Franosch@physik.tu-muenchen.de)

This file is part of the GNUSSL software package.  This package is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.  This software is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this distribution; if not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _gnussl_alloc_h_
#define _gnussl_alloc_h_


#include <memory>
#include <iostream>




// #if ( defined(INTEL_CPP) || ( __GNUC__ >= 3 && __GNUC_MINOR__ >= 2 ) )
template <class _Tp>
inline void gnussl_destroy(_Tp* __pointer) {
  __pointer->~_Tp();
}

template <class _T1, class _T2>
inline void gnussl_construct(_T1* __p, const _T2& __value) {
  new (__p) _T1(__value);
}

template <class _T1>
inline void gnussl_construct(_T1* __p) {
  new (__p) _T1();
}
// #endif




template <class T>
inline T* gnussl_allocate(ptrdiff_t size, T*) {
    std::set_new_handler(0);
    T* tmp = reinterpret_cast<T*>(::operator new(size * sizeof(T)));
    if (tmp == 0) {
	std::cerr << "out of memory" << std::endl; 
	exit(1);
    }
    return tmp;
}


template <class T>
inline void gnussl_deallocate(T* buffer) {
    ::operator delete(buffer);
}


template <class T>
class gnussl_allocator {
public:
    typedef T value_type;
    typedef T* pointer;
    typedef const T* const_pointer;
    typedef T& reference;
    typedef const T& const_reference;
    typedef size_t size_type;
    typedef ptrdiff_t difference_type;
    pointer allocate(size_type n) { 
	return gnussl_allocate(static_cast<difference_type>(n),
			       reinterpret_cast<pointer>(0));
    }
    void deallocate(pointer p) { gnussl_deallocate(p); }
  /*
    pointer address(reference x) { return (pointer)&x; }
    const_pointer const_address(const_reference x) { 
	return (const_pointer)&x; 
    }
    size_type init_page_size() { 
	return max(size_type(1), size_type(4096/sizeof(T))); 
    }
    size_type max_size() const { 
	return max(size_type(1), size_type(UINT_MAX/sizeof(T))); 
    }
  */
};


class gnussl_allocator<void> {
public:
    typedef void* pointer;
};


#endif // _gnussl_alloc_h_
