// mathematical extensions

/*
J.M.P. Franosch (Jan-Moritz.Franosch@physik.tu-muenchen.de)

This file is part of the GNUSSL software package.  This package is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.  This software is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this distribution; if not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _mathext_h_
#define _mathext_h_


#include <cstdlib>
#include <cmath>




#ifdef INTEL_CPP
inline double abs (double x) { return fabs (x); }
inline long double abs (long double x) { return fabs (x); }
#endif


const long double pi=3.14159265358979323846264338327;


inline double equal_rand() {
  return ( rand()+double(RAND_MAX)*double(rand()) )/ 
    ( double(RAND_MAX)*double(RAND_MAX) );
}


inline double equal_rand(const double x1, const double x2) {
  return x1+equal_rand()*(x2-x1);
}

inline int equal_rand(const int i1, const int i2) {
  int die=rand();
  while(die>RAND_MAX-RAND_MAX%(i2-i1+1)) {
    die=rand();
  }
  return (i1+die%(i2-i1+1));
}

inline void rand_seed(unsigned s) {
  srand(s);
}


inline double sign(const double x) {
  return x==0 ? 0 : (x>0 ? 1 : -1);
}


inline double sqr(const double& x) {
  return x*x;
}

inline double heaviside(const double& x) {
  return ( x>0.0 ? double(1.0) : double(0.0) );
}

inline double rectify(const double& x) {
  return ( x>0.0 ? x : 0 );
}


inline double gauss_rand(const double sigma) {
  static int iset=0;
  static double gset;
  double fac, rsq, v1, v2;
  if (iset==0) {
    do {
      v1=2*equal_rand()-1;
      v2=2*equal_rand()-1;
      rsq=sqr(v1)+sqr(v2);
    } while (rsq>=1 || rsq==0);
    fac=sqrt(-2*log(rsq)/rsq);
    gset=sigma*v1*fac;
    iset=1;
    return sigma*v2*fac;
  } else {
    iset=0;
    return gset;
  }
}


inline double rad_to_deg(const double angle) {
  return 180*(angle/pi);
}




#endif // _mathext_h_
