// tests convolution.h


/*
J.M.P. Franosch (Jan-Moritz.Franosch@physik.tu-muenchen.de)

This file is part of the GNUSSL software package.  This package is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.  This software is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this distribution; if not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <limits>
#include <convolution.h>
#include <mathext.h>




template<class Real, class S1, class S2, class S3>
mVector<Real,S3>& 
convolution_without_fft(const mVector<Real,S1>& function, const mVector<Real,S2>& kernel,
			mVector<Real,S3>& conv) {
// conv(i)=sum( function(j)*kernel(i-j+1), j=-infinity...infinity )
  for (size_t i=1; i<=conv.size(); i++) {
    conv(i)=0;
    for (size_t j=( i>kernel.size() ? i-kernel.size()+1 : 1); j<=i; j++) {
      conv(i)+=function(j)*kernel(i-j+1);
    };
  };
  return conv;
};


template<class Real>
void test() {
  for (size_t n=1; n<=1000000; n+=(n<=20 ? 1 : 1+2*n) ) {

    cout << "n=" << n << endl;

    for (size_t m=1; m<=(n<=100 ? n+1 : 100); m+=(n<=20 ? 1 : 1+2*m) ) {

      mVector<float> function(n);
      mVector<float> kernel(m);
      for (size_t i=1; i<=n; i++) {
	function(i)=frand(-1,1);
      };
      for (size_t i=1; i<=m; i++) {
	kernel(i)=frand(-1,1);
      };

      mVector<float> result(n);
      convolution(function,kernel,result);

      mVector<float> result2(n);
      convolution_without_fft(function,kernel,result2);

      float fault=sup(result2-result);
      if ( fault > 100*numeric_limits<float>::epsilon() ) {
	cerr << "error: solution incorrect, fault=" 
	     << fault << ", epsilon=" << numeric_limits<float>::epsilon() << endl;
      };

    };
  };  
};


main() {
  test<float>();
  test<double>();
};

