// tests eigenvectors.h


/*
J.M.P. Franosch (Jan-Moritz.Franosch@physik.tu-muenchen.de)

This file is part of the GNUSSL software package.  This package is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.  This software is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this distribution; if not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <eigenvectors.h>
#include <mathext.h>
#include <limits>




template<class Real, class S1, class S2, class S3>
void test_eigenvectors(const mMatrix<complex<Real>,S1>& M, 
		       const mMatrix<complex<Real>,S2>& EV,
		       const mVector<complex<Real>,S3>& w) {
  mVector< complex<Real> > z(w.size());
  for (size_t i=1; i<=w.size(); i++) {
    Real test=sup(dot(M,EV.col(i),z)-w(i)*EV.col(i));
    if ( test > 100*numeric_limits<Real>::epsilon() ) {
      cerr << "error: eigenvector " << i << " incorrect, test=" 
	   << test << ", epsilon=" << numeric_limits<Real>::epsilon() << endl;
    };
  };
};


template<class Complex>
void test_complex() {
  for (int n=2; n<=20; n++) {

    cout << "n=" << n << endl;

    mMatrix< Complex > M(n,n);
    mMatrix< Complex > EV(n,n);
    mVector< Complex > w(n);

    for (int i=1; i<=n; i++) {
      for (int j=1; j<=n; j++) {
	M(i,j)=Complex(frand(-1,1));
      };
    };
    eigenvectors(M,EV,w);
    test_eigenvectors(M,EV,w);

    for (int i=1; i<=n; i++) {
      for (int j=1; j<=n; j++) {
	M(i,j)=Complex(frand(-1,1),frand(-1,1));
      };
    };
    eigenvectors(M,EV,w);
    test_eigenvectors(M,EV,w);

    // also test some special cases
    if (n==1) {
      M(1,1)=Complex(1);
      eigenvectors(M,EV,w);
      test_eigenvectors(M,EV,w);      

      M(1,1)=Complex(0);
      eigenvectors(M,EV,w);
      test_eigenvectors(M,EV,w);      
    };
    if (n==2) {
      M(1,1)=Complex(1,0.0);
      M(2,2)=Complex(1,0.0);
      M(1,2)=Complex(2,0.0);
      M(2,1)=Complex(-2,0.0);
      eigenvectors(M,EV,w);
      test_eigenvectors(M,EV,w);      

      M=Complex(0);
      M.diagonal()=Complex(1);
      eigenvectors(M,EV,w);
      test_eigenvectors(M,EV,w);      

      M=Complex(0);
      eigenvectors(M,EV,w);
      test_eigenvectors(M,EV,w);      
    };
    if (n==3) {
      M=Complex(0);
      M.diagonal()=Complex(1);
      eigenvectors(M,EV,w);
      test_eigenvectors(M,EV,w);      

      M=Complex(0);
      eigenvectors(M,EV,w);
      test_eigenvectors(M,EV,w);
    };


  };
};


main() {
  test_complex< complex<double> >();
  test_complex< complex<float> >();
};
