// tests least_squares.h


/*
J.M.P. Franosch (Jan-Moritz.Franosch@physik.tu-muenchen.de)

This file is part of the GNUSSL software package.  This package is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.  This software is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this distribution; if not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <least_squares.h>
#include <cmath>
#include <vector>
#include <mathext.h>




template<class iterator, class basisfunction_cla, class Real, class S1, class S2>
void test_solution(const iterator& begin,
		   const mVector<Real,S1>& y,
		   const basisfunction_cla& basis_function,
		   mVector<Real,S2>& a) {
  mVector<Real> d(y.size());
  for ({size_t i=1; iterator p=begin; } i<=d.size(); i++, p++) {
    mVector<Real> b(a.size());
    for (size_t j=1; j<=b.size(); j++) {
      b(j)=basis_function(j,*p);
    };
    d(i)=sum(a*b);
  };
  Real fault=sup(d-y);
  if ( fault > 100*numeric_limits<Real>::epsilon() ) {
    cerr << "error: solution incorrect, fault=" 
	 << fault << ", epsilon=" << numeric_limits<Real>::epsilon() << endl;
  };
};

template<class Real>
class basis_function_cla {
public:
  Real operator()(const int n, const Real x) const {
    return pow(x,n-1);
  };
};


template<class Real>
void test() {
  for (int m=1; m<=10; m++) {

    cout << "m=" << m << endl;
    
    for (int n=m; n<=50; n++) {
      
      vector<Real> v;
      mVector<Real> y(n);
      mVector<Real> a(m);

      for (int i=1; i<=m; i++) {
	a(i)=frand(-1,1);
      };
      for (int i=1; i<=n; i++) {
	Real x=frand(-1,1);
	mVector<Real> b(a.size());
	for (size_t j=1; j<=b.size(); j++) {
	  b(j)=basis_function_cla<Real>()(j,x);
	};
	v.push_back(sum(a*b));
      };
      
      least_squares(v.begin(),y,basis_function_cla<Real>(),a);
      test_solution(v.begin(),y,basis_function_cla<Real>(),a);
      
    };
  };
};


main() {
  test<double>();
  test<float>();
};
