// tests optimize.h


/*
J.M.P. Franosch (Jan-Moritz.Franosch@physik.tu-muenchen.de)

This file is part of the GNUSSL software package.  This package is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.  This software is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this distribution; if not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <vector>
#include <optimize.h>
#include <mathext.h>




class quadratic_function_cla {

public:

  double operator()(const mVector<double>& x) const {
    double r=0;
    for (size_t i=1; i<=x.size(); i++) {
      r+=(x(i)-1)*(x(i)-1);
    };
    return r+1;
  };

};


class function_cla {
private:
  mMatrix<double> T;
public:
  function_cla(const int n) : T(n,n) {
    for (int i=1; i<=n; i++) {
      for (int j=1; j<=n; j++) {
	T(i,j)=frand(-1,1);
      };
    };
  };
  double operator()(const mVector<double>& x) const {
    mVector<double> x_(x.size());
    dot(T,x-1.0,x_);
    double r=0;
    for (size_t i=1; i<=x.size(); i++) {
      r+=x_(i)*x_(i);
      r+=1-cos(x_(i));
    };
    return r+1;
  };
};


main() {

  // optimize_downhill_simplex
  /*
  for (int n=2; n<=40; n++) {

    cout << "n=" << n << ", ";

    mMatrix<double> p(n+1,n);
    mVector<double> y(n+1);
    mVector<double> x(n);

    //    quadratic_function_cla f;
    function_cla f(n);

    // starting point P_n+1=0, P_i=e_i
    for (int i=1; i<=n+1; i++) {
      for (int j=1; j<=n; j++) {
	x(j)=10*frand(0.1,1)*(j==i);
	p(i,j)=x(j);
	y(i)=f(x);
      };
    };

    int nfunk=0;
    optimize_downhill_simplex(p,y,f,nfunk,0.00001);
    
    x=p.row(1);
    cout << "nfunk=" << nfunk << ", f_min=" << f(x) << endl;

    p.row(1)-=1.0;
    double test=max(double(sqr(sup(p.row(1)))),abs(y(1)-1));

    if ( test>1000*numeric_limits<double>::epsilon() ) {
//      cerr << "error: solution incorrect, test=" 
//	   << test << ", epsilon=" << numeric_limits<double>::epsilon() << endl;
    };

  };
*/

  // optimize_quadratic_approximation
  /*
  for (int n=2; n<=20; n++) {

    cout << "n=" << n << endl;

    mVector<double> x0(n);
    mVector<double> x_min(x0.size());
    x0=2.0;
    function_cla f(x0.size());
    int nfunk=0;
    optimize_quadratic_approximation(x0,10.0,f,x_min,nfunk,0.0001);

    cout << "nfunk=" << nfunk << ", x_min=" << x_min << endl;

  };
  */


  for (int n=2; n<=100; n+=1) {

    cout << "n=" << n << ", ";

    mVector<double> x0(n);
    mVector<double> x_min(x0.size());
    x0=2.0;

    //    quadratic_function_cla f;
    function_cla f(n);

    int nfunk=0;
    optimize_Powell(x0,f,nfunk,0.00001);

    cout << "nfunk=" << nfunk << ", f_min=" << f(x0) << endl;

  };


};
