// tests solve_linear.h


/*
J.M.P. Franosch (Jan-Moritz.Franosch@physik.tu-muenchen.de)

This file is part of the GNUSSL software package.  This package is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.  This software is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this distribution; if not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <solve_linear.h>
#include <mathext.h>




template<class Real, class S1, class S2, class S3>
void test_solution(const mMatrix<Real,S1>& A, 
		   const mVector<Real,S2>& b,
		   const mVector<Real,S3>& x) {
  mVector<Real> z(b.size());
  Real fault=sup(dot(A,x,z)-b);
  if ( fault > 10000*numeric_limits<Real>::epsilon() ) {
    cerr << "error: solution incorrect, fault=" 
	 << fault << ", epsilon=" << numeric_limits<Real>::epsilon() << endl;
  };
};


template<class Real>
void test() {
  for (int n=1; n<=50; n++) {

    cout << "n=" << n << endl;

    mMatrix<Real> A(n,n);
    mVector<Real> b(n), x(n);

    for (int i=1; i<=n; i++) {
      for (int j=1; j<=n; j++) {
	A(i,j)=Real(frand(-1,1));
      };
      b(i)=Real(frand(-1,1));
    };

    solve_linear(A,b,x);
    test_solution(A,b,x);

    // also test some special cases
    if (n==1) {
      A(1,1)=Real(1);
      b(1)=2;
      solve_linear(A,b,x);
      test_solution(A,b,x);

      A(1,1)=Real(1);
      b(1)=0;
      solve_linear(A,b,x);
      test_solution(A,b,x);
    };
    if (n>=2 && n<=5) {
      A=Real(0);
      A.diagonal()=Real(1);
      solve_linear(A,b,x);
      test_solution(A,b,x);
    };
  };
};


main() {
  test<double>();
  test<float>();
};
