/***************************************************************************
                          sbool.cpp  -  description
                             -------------------
    begin                : Thu Dec 12 2002
    author               : Christof Schatz, Moritz Franosch
    copyright            : (C) 2002 by ASKOS http://www.ASKOS.de
    email                : info@askos.de
 ***************************************************************************/

#include "askosio/errorcheck.h"

#include "sbool.h"


using namespace std;


const Sbool Sbool::missing=Sbool(Sbool::missing_v);

const Sbool Sbool::undefined=Sbool(Sbool::undefined_v);


Sbool::Sbool(const Sbool::value_t v) : 
  value(v) {
}


Sbool::Sbool() : 
  value(undefined_v) {
}


Sbool::Sbool(const bool b) : 
  value(b ? true_v : false_v) {
}


Sbool::Sbool(const Sbool& sbool) : 
  value(sbool.value) {
}


bool Sbool::is_true() const {
  return value==true_v;
}


bool Sbool::is_false() const {
  return value==false_v;
}


bool Sbool::is_missing() const {
  return value==missing_v;
}


bool Sbool::is_undefined() const {
  return value==undefined_v;
}


bool Sbool::is_valid() const {
  return !is_missing() && !is_undefined();
}


Sbool& Sbool::operator=(const Sbool& sbool) {
  value=sbool.value;
  return *this;
}


const Sbool Sbool::operator!() const {
  if ( is_undefined() ) {
    return undefined;
  }
  if ( is_missing() ) {
    return missing;
  }
  if ( is_true() ) {
    return Sbool(false);
  }
  return Sbool(true);
}


Sbool::operator bool() const {
  return is_true();
}


const Sbool operator&&(const Sbool& sbool1, const Sbool& sbool2) {
  if ( sbool1.is_undefined() || sbool2.is_undefined() ) {
    return Sbool::undefined;
  }
  // Keiner der Operanden ist undefined.
  if ( sbool1.is_missing() || sbool2.is_missing() ) {
    // Mindestens einer der Operanden ist missing, das Ergebnis von
    // "und" ist dann missing.
    return Sbool::missing;
  }
  // Keiner der Operanden ist missing, das Ergebnis reduziert sich auf
  // zweiwertige Logik.
  return Sbool(sbool1.is_true() && sbool2.is_true());
}


const Sbool operator||(const Sbool& sbool1, const Sbool& sbool2) {
  if ( sbool1.is_undefined() || sbool2.is_undefined() ) {
    return Sbool::undefined;
  }
  // Keiner der Operanden ist undefined.
  if ( sbool1.is_true() || sbool2.is_true() ) {
    // Mindestens einer der Operanden ist true, der andere nicht
    // undefined. "Oder" ergibt dann in jedem Fall true.
    return Sbool(true);
  }
  if ( sbool1.is_missing() || sbool2.is_missing() ) {
    // Mindestens einer der Operanden ist missing. Keiner ist true,
    // der andere ist also missing oder false. "Oder" ergibt dann in
    // jedem Fall missing (missing oder missing = missing, missing
    // oder false = missing).
    return Sbool(false);
  }
  // Keiner der Operanden ist missing, das Ergebnis reduziert sich auf
  // zweiwertige Logik.
  return Sbool(sbool1.is_true() || sbool2.is_true());
}


const Sbool operator==(const Sbool& sbool1, const Sbool& sbool2) {
  if ( sbool1.is_undefined() || sbool2.is_undefined() ) {
    return Sbool::undefined;
  }
  // Keiner der Operanden ist undefined.
  if ( sbool1.is_missing() || sbool2.is_missing() ) {
    // Mindestens einer der Operanden ist missing, das Ergebnis von
    // "gleich" ist dann missing.
    return Sbool::missing;
  }
  // Keiner der Operanden ist missing, das Ergebnis reduziert sich auf
  // zweiwertige Logik.
  return Sbool( sbool1.is_true() == sbool2.is_true() );
}


ostream& operator<<(ostream& o, const Sbool& sbool) {
  if ( sbool.is_undefined() ) {
    o << "undefined";
    return o;
  }
  if ( sbool.is_missing() ) {
    o << "missing";
    return o;
  }
  if ( sbool.is_true() ) {
    o << "true";
    return o;
  }
  if ( sbool.is_false() ) {
    o << "false";
    return o;
  }
  FATAL_ERROR;
  return o;
}
