/***************************************************************************
                          date.cpp  -  description
                             -------------------
    begin                : Thu Nov 28 2002
    author               : Moritz Franosch
    copyright            : (C) 2002 by ASKOS http://www.ASKOS.de
    email                : mail@Franosch.org
 ***************************************************************************/

#include <cmath>

#include "askosio/errorcheck.h"

#include "date.h"


using namespace std;




Date::Date(const int year, const int month, const int day) :
  tm_time(),
  time(0) // desgl. time 
{
  set_date(year,month,day,0,0,0); // 0.00 Uhr, 0 Sekunden
}


Date::Date(const int year, const int month, const int day,
  const int hours, const int minutes, double seconds) :
  tm_time(),
  time(0) // desgl. time 
{
  set_date(year,month,day,hours,minutes,seconds);
}


Date::Date(double seconds) :
  tm_time(),
  time(seconds) // time ist die UNIX-Zeit 
{
  // setze tm_time auf die UNIX-Zeit seconds
  time_t t=static_cast<time_t>(floor(seconds));
  // wandle in eine struct tm um
  tm* tm_time_ptr = localtime(&t);
  // tm_time_ptr==0 wenn die Konversion scheitert
  if (tm_time_ptr==0) {
    FATAL_ERROR_TEXT("Invalid date.");
    /*
    set_date(0,0,0,0,0,0); // Datum auf das Jahr 0 setzen.
    return;
    */
  }
  tm_time=*tm_time_ptr;
}


void Date::set_date(const int year, const int month, const int day,
		    const int hours, const int minutes, double seconds) {
  // Initialisiere tm_time auf die UNIX-Zeit 0 Sekunden.
  time_t t=0;
  // wandle in eine struct tm um
  tm* tm_time_ptr = localtime(&t);
  // tm_time_ptr==0 wenn die Konversion scheitert
  if ( tm_time_ptr==0 ) {
    FATAL_ERROR_TEXT("localtime(0) failed!?");
  }
  tm_time = *tm_time_ptr;

  tm_time.tm_year=year-1900; // tm verlangt das Jahr seit 1900.
  tm_time.tm_mon=month-1; // tm verlangt die Monate von 0 bis 11.
  tm_time.tm_mday=day; // Tag von 1 bis 31 is korrekt fuer tm.
  tm_time.tm_hour=hours;
  tm_time.tm_min=minutes;
  tm_time.tm_sec=static_cast<int>(floor(seconds)); // ganze Sekunden
  if ( mktime(&tm_time)==-1 ) { // Restliche Felder von tm_time ausfuellen.
    // ist gescheitert
    FATAL_ERROR_TEXT("Invalid date.");
  }
}


int Date::get_year() const {
  // tm enthaelt das Jahr seit 1900, wir geben das absolute
  // Kalenderjahr zurueck.
  return tm_time.tm_year+1900;
}


int Date::get_month() const {
  // tm enthaelt den Monat im Bereich 0 bis 11,
  // wir geben den Monat im bereich 1 bis 12 zurueck.
  return tm_time.tm_mon+1;
}


int Date::get_day() const {
  // Tag im Monat.
  return tm_time.tm_mday;
}


int Date::get_hours() const {
  return tm_time.tm_hour;
}


int Date::get_minutes() const {
  return tm_time.tm_min;
}


double Date::get_seconds() const {
  // Zahl der vollen Sekunden seit Beginn der Minute plus Bruchteile
  // von Sekunden.
  return tm_time.tm_sec+(time-floor(time));
}


double Date::get_time() const {
  return time;
}


const tm& Date::get_tm() const {
  return tm_time;
}


int Date::get_weekday() const {
  // tm enthaelt 0 fuer Sonntag,
  // wir zaehlen den Sonntag als 7.
  if ( tm_time.tm_wday==0 ) {
    return 7;
  }
  return tm_time.tm_wday;
}


bool Date::is_holiday() const {
  //// fixme
}
