/***************************************************************************
                          date.h  -  description
                             -------------------
    begin                : Thu Nov 28 2002
    author               : Moritz Franosch
    copyright            : (C) 2002 by ASKOS http://www.ASKOS.de
    email                : mail@Franosch.org
 ***************************************************************************/

#ifndef DATE_H
#define DATE_H


// Borland C++ braucht time.h statt ctime.
#include <time.h>




/** Datum.
 *
 *  Wandelt die UNIX-Zeit (Sekunden seit 00:00:00, 1. Januar 1970, UTC)
 *  in die "normale" Kalenderzeit um und umgekehrt.
 *
 *  Gibt den Wochentag an.
 */
class Date {

private:

  tm tm_time; // Datum in "aufgesplitteter" Form (nach ISO C).
  double time; // Datum in Sekunden als UNIX-Zeit.
  bool holiday; // Der Tag ist ein deutscher Feiertag.

  /** Das Datum wird auf Jahr (seit Beginn der Zeitrechnung), Monat
   *  (1...12), Tag (1...31), Stunden (0...23), Minuten (0...59),
   *  Sekunden [0,60[ gesetzt.
   */
  void set_date(const int year, const int month, const int day,
		const int hours, const int minutes, double seconds);

public: 

  /** Das Datum wird auf Jahr (seit Beginn der Zeitrechnung), Monat
   *  (1...12), Tag (1...31), 0.00 Uhr, 0 Sekunden gesetzt.
   */
  Date(const int year, const int month, const int day);
  
  /** Das Datum wird auf Jahr (seit Beginn der Zeitrechnung), Monat
   *  (1...12), Tag (1...31), Stunden (0...23), Minuten (0...59),
   *  Sekunden [0,60[ gesetzt.
   */
  Date(const int year, const int month, const int day,
       const int hours, const int minutes, double seconds);

  /** Datum wird mit der Zahl der UNIX-Zeit initialisiert.
   */
  explicit Date(double seconds);

  /** Zahl der vollen Kalender-Jahre seit Beginn der Zeitrechnung.
   */
  int get_year() const;

  /** Monat im Jahr, von 1 bis 12. 
   */
  int get_month() const;

  /** Tag im Monat, von 1 bis 31. 
   */
  int get_day() const;

  /** Zahl der vollen Stunden nach Mitternacht, von 0 bis 23.
   */
  int get_hours() const;

  /** Zahl der vollen Minuten seit Beginn der Stunde, von 0 bis 59.
   */
  int get_minutes() const;

  /** Sekunden seit Beginn der Minute und Bruchteile von Sekunden.
   */
  double get_seconds() const;

  /** UNIX-Zeit in Sekunden.
   */
  double get_time() const;

  /** Datum in "aufgesplitteter" Form als struct tm (nach ISO C).
   */
  const tm& get_tm() const;

  /** True, falls der Tag ein deutscher Feiertag ist, sonst false.
   */
  bool is_holiday() const;

  /** Wochentag: Montag=1, Sonntag=7. 
   */
  int get_weekday() const;

};


#endif // DATE_H
