/***************************************************************************
                          casc.h  -  description
                             -------------------
    begin                : Mon Dec 24 2001
    copyright            : (C) 2001 by Moritz Franosch
    email                : mail@Franosch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CASC_H
#define CASC_H


#include <iostream>
#include <list>
#include <string>
#include <sstream>

#include "casc_map.h"


using namespace std;


class Casc {

private:

  istream& in;
  Casc_Map<string> strings;
  Casc_Map<double> doubles;
  Casc_Map<int> ints;
  Casc_Map< list<string> > string_lists;
  Casc_Map< list<double> > double_lists;
  Casc_Map< list<int> > int_lists;
  bool parsed;
  enum{double_type,int_type,ascii_type,undefined_type} type;
  static string format_directory;

protected:

  template<class C>
  const C& get(const Casc_Map<C>& casc_map, const string& name) const {
    if ( !casc_map.has(name) ) {
      error("No such element '"+name+"'.");
    }
    return casc_map.get(name);
  }

  void error(const string& message) const;

  template<class C>
  void read(const string& parameter, const string& line, Casc_Map<C>& casc_map) {
    if ( casc_map.has(parameter) ) {
      error("Parameter '"+parameter+"' already defined.");
    }
    //    istrstream instr(line.c_str());
    istringstream instr(line);
    C p;
    if ( !(instr >> p) ) {
      error("Bad parameter '"+parameter+" "+line+"'.");
    }
    instr >> ws;
    if ( !instr.eof() ) {
      error("Bad parameter '"+parameter+" "+line+"'.");
    }
    casc_map.add(parameter,p);
  }

  template<class C>
  void read_list(const string& parameter, const string& line, 
		 Casc_Map< list<C> >& casc_map) {
    if ( casc_map.has(parameter) ) {
      error("Parameter '"+parameter+"' already defined.");
    }
    //    istrstream instr(line.c_str());
    istringstream instr(line);
    C p;
    list<C> p_list;
    while ( (instr >> p) ) {
      p_list.push_back(p);
    }
    casc_map.add(parameter,p_list);
    instr >> ws;
    if ( !instr.eof() ) {
      error("Bad parameter '"+parameter+" "+line+"'.");
    }
  }

public:

  Casc(istream& in_ini);

  ~Casc();

  static void set_format_directory(const string& directory);

  void parse(const string& format="");

  const string& get_string(const string& name) const;

  const double& get_double(const string& name) const;

  const int& get_int(const string& name) const;

  const list<double>& get_double_list(const string& name) const;

  const list<int>& get_int_list(const string& name) const;

  double read();

  int read_int();

};




#endif // CASC_H
