/***************************************************************************
                          commandline.h  -  description
                             -------------------
    begin                : Sun Mar 18 2001
    copyright            : (C) 2001 by Moritz Franosch
    email                : mail@franosch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef COMMANDLINE_H
#define COMMANDLINE_H


#include <argp.h>

#include <string>
#include <list>
#include <vector>
#include <map>


using namespace std;


error_t parse_opt(int key, char* argument, argp_state* state);


class Commandline {

private:

  string myname, mydir;
  string version;
  map<string,string> strings;
  map<string,int> ints;
  map<string,double> doubles;
  map<string,bool> switches;
  map< string, list<double> > lists_double;
  map< string, list<string> > lists_string;
  vector<string> filenames;
  const struct argp_option* options;

  Commandline(const Commandline&);

  Commandline& operator=(const Commandline&);

  template<class T>
  bool specified(const string& s, const map<string,T>& m) const;

  template<class T>
  void check_specified(const string& s, const map<string,T>& m) const;

  template<class T>
  void check_not_specified(const char* s, const map<string,T>& m, 
			   argp_state* state);

  void err_argument_not_specified(const string& s) const;

  void err_argument_specified(const string& s, argp_state* state);

  void set_string(const char* s, const char* p, argp_state* state);

  void set_int(const char* s, const char* p, argp_state* state);

  void set_double(const char* s, const char* p, argp_state* state);

  void set_switch(const char* s, argp_state* state);

  void set_list_double(const char* s, const char* p, argp_state* state);

  void set_list_string(const char* s, const char* p, argp_state* state);

public:

  Commandline(const int argc, char *argv[],
	      const char* version_ini, const char* bug_address,
	      const char* doc, const char* args_doc,
	      const struct argp_option options_ini[]);

  bool specified_string(const string& s);

  bool specified_int(const string& s);

  bool specified_double(const string& s);

  bool specified_list_double(const string& s);

  bool specified_list_string(const string& s);

  const string& get_string(const string& s);

  const string& get_string(const string& s, const string& d);

  int get_int(const string& s);

  int get_int(const string& s, const int d);

  double get_double(const string& s);

  double get_double(const string& s, const double d);

  bool get_switch(const string& s);

  const list<double>& get_list_double(const string& s);

  const list<string>& get_list_string(const string& s);

  size_t get_nfilenames() const;

  /** Get the input filename number i (i=0...).
   */
  const string& get_filename(const size_t i) const;

  const string& get_myname() const;

  const string& get_mydir() const;

  const string& get_version() const;

  error_t parse(int key, char* argument, argp_state* state);

};


template<class T>
bool Commandline::specified(const string& s, const map<string,T>& m) const {
  return m.find(s)!=m.end();
}


template<class T>
void Commandline::check_specified(const string& s, const map<string,T>& m) const {
  if (s=="") {
    cerr << get_myname() << ": Program bug. String s is empty." << endl;
  }
  if ( !specified(s,m) ) {
    err_argument_not_specified(s);
  }
}


template<class T>
void Commandline::check_not_specified(const char* s, const map<string,T>& m, 
				      argp_state* state) {
  if (s==0) {
    cerr << get_myname() << ": Program bug. String s is NULL pointer." << endl;
  }
  if ( specified(string(s),m) ) {
    err_argument_specified(s,state);
  }
}




#endif // COMMANDLINE_H
