/***************************************************************************
                          log.cpp  -  description
                             -------------------
    begin                : Tue Apr 9 2002
    author               : Moritz Franosch
    copyright            : (C) 2002 by ASKOS http://www.ASKOS.de
    email                : mail@Franosch.org
 ***************************************************************************/

#include <string>

#include "askosdates/convert_date.h"

#include "askos_fatal_exception.h"
#include "log.h"
#include "nullostream.h"


using namespace std;


Logdevice Log::default_logdevice; // statische Variable initialisieren

Nullostream Log::nullostream; // statische Variable initialisieren


Logdevice& Log::get_default_logdevice() {
  return default_logdevice;
}


string Log::get_classname(const string& function_name) {
  string::size_type index=function_name.find("::");
  if (index==string::npos) {
    // nicht gefunden
    return "";
  }
  string::size_type first_whitespace=function_name.rfind(' ',index);
  string::size_type first_char=0;
  if ( first_whitespace!=string::npos ) {
    first_char = first_whitespace+1;
  }
  return function_name.substr(first_char,index-first_char);
}


string Log::get_methodname(const string& function_name) {
  string::size_type index=function_name.find("::");
  if (index==string::npos) {
    // nicht gefunden
    return function_name;
  }
  return function_name.substr(index+2);
}


Log::Log(Logdevice* logdevice_ptr, const string& classname_ini,
	 const string& methodname_ini) :
  logdevice(*logdevice_ptr),
  classname(classname_ini), methodname(methodname_ini),
  class_method(classname+"::"+methodname),
  output_ptr(&(logdevice_ptr->get_ostream(classname))),
  depth(default_logdevice.get_depth()),
  indentation("  "),
  log_debug(default_logdevice.debug(classname)) {
  logdevice.increase_depth();
  if (log_debug) {
    debug() << "Enter." << endl;
  }
}


Log::Log(const string& classname_ini, const string& methodname_ini) :
  logdevice(default_logdevice),
  classname(classname_ini), methodname(methodname_ini),
  class_method(classname+"::"+methodname),
  output_ptr(&default_logdevice.get_ostream(classname)),
  depth(default_logdevice.get_depth()),
  indentation("  "),
  log_debug(default_logdevice.debug(classname)) {
  logdevice.increase_depth();
  if (log_debug) {
    debug() << "Enter." << endl;
  }
}


Log::Log(const string& function_name) :
  logdevice(default_logdevice),
  classname(get_classname(function_name)), methodname(get_methodname(function_name)),
  class_method(function_name),
  output_ptr(&default_logdevice.get_ostream(classname)),
  depth(default_logdevice.get_depth()),
  indentation("  "),
  log_debug(logdevice.debug(classname)) {

  logdevice.increase_depth();
  if (log_debug) {
    debug() << "Enter." << endl;
  }
}


Log::Log() :
  logdevice(default_logdevice),
  classname(""), methodname(""),
  class_method(""),
  output_ptr(&default_logdevice.get_ostream(classname)),
  depth(logdevice.get_depth()),
  indentation("  "),
  log_debug(false) {
  logdevice.increase_depth();
}


Log::~Log() {
  if (log_debug) {
    debug() << "Exit." << endl;
  }
  logdevice.decrease_depth();
}


ostream& Log::internal_error() {
  return header(*output_ptr,"INTERNAL ERROR");
}


ostream& Log::error() {
  return header(*output_ptr,"ERROR");
}


ostream& Log::warn() {
  return header(*output_ptr,"Warning");
}


ostream& Log::info() {
  return header(*output_ptr,"Info");
}


ostream& Log::debug() {
  if (log_debug) {
    return header(*output_ptr,"Debug");
  }
  return nullostream;
}


void Log::error_exit() {
  warn() << "Exiting with FAILURE." << endl;
  exit(EXIT_FAILURE);
}


void Log::fatal_error(const string& message) {
  throw Askos_Fatal_Exception(message);
}


ostream& Log::header(ostream& out, const string& text) {
  if ( logdevice.get_log_time() ) {
    out << time_to_date(time(0));
  }
  for (int i=0; i<depth; i++) {
    out << indentation;
  }
  out << text << ": ";
  if (log_debug) {
    out << class_method << ": ";
  }
  return out;
}
