/***************************************************************************
                          logdevice.h  -  description
                             -------------------
    begin                : Tue Apr 9 2002
    author               : Moritz Franosch
    copyright            : (C) 2002 by ASKOS http://www.ASKOS.de
    email                : mail@Franosch.org
 ***************************************************************************/

#ifndef LOGDEVICE_H
#define LOGDEVICE_H

#include <string>
#include <iostream>
#include <set>

#include "multiple_streambuf.h"


using namespace std;


class Logdevice {

private:

  int depth; // indentation
  bool log_time; // whether a timestamp should be logged

  Multiple_Streambuf logstreambuf;
  // streambuf for simultaneous logging in multiple streams

  ostream out; // outputstream for logging

  set<string> debug_classnames;

public:

  Logdevice();

  //  bool open(const string& filename);

  /*The logdevice logs to the stream outputstream.
   */
  void connect_to_stream(ostream& outputstream);

  /* Adds the streambuf of the outputstream to the log devices.
   */
  void add_stream(ostream& outputstream);

  /* Adds the streambuf of the outputstream to the log devices.
   */
  void add_stream(Pointer<ostream> ostream_ptr);

  bool debug(const string& classname) const;

  void debug_on(const string& classname);

  ostream& get_ostream(const string& classname);

  // streambuf* get_debugstreambuf(const string& classname);

  int get_depth() const {
    return depth;
  }

  void increase_depth() {
    depth++;
  }

  void decrease_depth() {
    if (depth==0) {
      cerr << "Fatal Error: Logdevice::decrease_depth(): depth==0" << endl;
      return;
    }
    depth--;
  }

  /* Returs whether a timestamp should be logged.
   */
  bool get_log_time() const;

  /* Sets whether a timestamp should be logged.
   */
  void set_log_time(const bool log_time=true);

};


#endif // LOGDEVICE_H
