/***************************************************************************
                          string_convert.cpp  -  description
                             -------------------
    begin                : Wed Apr 24 2002
    author               : Moritz Franosch
    copyright            : (C) 2002 by ASKOS http://www.ASKOS.de
    email                : mail@Franosch.org
 ***************************************************************************/

#include <cctype>
#include <strstream>

#include "string_convert.h"




const string string_convert::int_to_zero_padded_string(const int i, const unsigned n) {
  string tmp_string=string_convert::to_string(i);
  if (tmp_string.size()<n) {
    tmp_string.insert(unsigned(0),n-tmp_string.size(),'0');
  }
  return tmp_string;
}


bool string_convert::is_int(const string& s) {
  istrstream input(s.c_str());
  int i=0;
  input >> i;
  input >> ws;
  return input.eof();
}


int string_convert::string_to_int(const string& s) {
  istrstream input(s.c_str());
  int i=0;
  input >> i;
  input >> ws;
  if ( input.eof() ) {
    return i;
  }
  return 0;
  /*
  // istringstream bei Borland C++ fehlerhaft
  istringstream input(s);
  int i=0;
  input >> i;
  return i;
  */
}


#ifdef __GNUC__
int64_t string_convert::string_to_int64(const string& s) {
#else
__int64 string_convert::string_to_int64(const string& s) {
#endif
  istrstream input(s.c_str());
#ifdef __GNUC__
  int64_t i=0;
#else
  __int64 i=0;
#endif
  input >> i;
  return i;
}


bool string_convert::string_to_double(const string& s, double& r) {
  if ( s.length()==0 ) {
    return false;
  }
  char* endptr;
  r=strtod(s.c_str(),&endptr);
  return endptr==s.c_str()+s.length();
  /*
  // istringstream bei Borland C++ fehlerhaft
  istringstream input(s);
  r=0;
  input >> ws >> r >> ws;
  return input.eof();
  */
  /*
  istrstream input(s.c_str());
  r=0;
  if ( !( input >> ws >> r >> ws ) ) {
    return false;
  }
  return input.eof();
  // input.eof() ist bei Borland C++ immer false.
  // bool(input) ist dagegen immer true!?
  */
}


const string string_convert::tolower(const string& s) {
  string r;
  for (size_t i=0; i<s.length(); i++) {
    r+=::tolower(s[i]);
  }
  return r;
}


const string string_convert::basename(const string& s) {
  string::size_type pos=s.rfind('/');
  if (pos==string::npos) {
    return s;
  }
  return s.substr(pos+1,s.size());
}


const string string_convert::dirname(const string& s) {
  string::size_type pos=s.rfind('/');
  if (pos==string::npos) {
    return ".";
  }
  return s.substr(0,pos);
}


const string string_convert::without_whitespace(const string& s) {
  string r;
  for (string::size_type i=0; i<s.size(); i++) {
    if ( !isspace(s[i]) ) {
      r+=s[i];
    }
  }
  return r;
}


const string string_convert::without_leading_trailing_whitespace(const string& s) {
  string::size_type pos1=s.size(), pos2=0;
  for (string::size_type i=0; i<s.size(); i++) {
    if ( !isspace(s[i]) ) {
      pos1=i;
      break;
    }
  }
  if ( s.size()>0 ) {
    for (string::size_type i=s.size(); i>0; i--) {
      if ( !isspace(s[i-1]) ) {
	pos2=i;
	break;
      }
    }
  }
  return s.substr(pos1,pos2-pos1);
}


const string string_convert::substitute(const string& substituent, 
					      const string& substitution, 
					      const string& s) {
  string r;
  string::size_type pos=0;
  string::size_type found_pos=0;
  while ( (found_pos=s.find(substituent,pos))!=string::npos ) {
    r+=s.substr(pos,found_pos-pos);
    r+=substitution;
    pos=found_pos+substituent.size();
  }
  r+=s.substr(pos,s.size());
  return r;
}


const string seconds_to_daytime(const double& t) {
  string str;
  int d=int(floor(t/(3600*24)));
  double r=t-d*(3600*24);
  int h=int(floor(r/3600));
  r-=h*3600;
  int min=int(floor(r/60));
  double s=r-min*60;
  if (d!=0) {
    str+=string_convert::to_string(d)+" d ";
  }
  str+=string_convert::int_to_zero_padded_string(h,2)+":"
    +string_convert::int_to_zero_padded_string(min,2)+" "
    +string_convert::to_string(s);
  return str;
}
