/***************************************************************************
                          askosmath.cpp  -  description
                             -------------------
    begin                : Thu Nov 28 2002
    author               : Moritz Franosch
    copyright            : (C) 2002 by ASKOS http://www.ASKOS.de
    email                : mail@Franosch.org
 ***************************************************************************/

#include <cmath>
#include <limits>
#include <algorithm>

#include "askosio/errorcheck.h"

#include "math.h"


using namespace std;




double similar_bits(const double a, const double b, double scaling) {
  double diff=abs(a-b);
  if ( diff==0 ) {
    return numeric_limits<double>::digits;
  }
  if (scaling==0) {
    scaling=max( abs(a), abs(b) );
  }
  return max(0.0,log(scaling/diff)/log(2.0));
  // Borland C++ hat kein log2
  //  return max(0.0,log2(scaling/diff));
}


bool similar(const double a, const double b, const double scaling) {
  return similar_bits(a,b,scaling)>numeric_limits<double>::digits/2;
}


int remainder(const int a, const int n) {
#ifdef ASKOS_ERRORCHECK
  if ( n<=0 ) {
    FATAL_ERROR_TEXT("n must be > 0");
  }
#endif
  // a % n funktioniert nicht, da das Vorzeichen von a % n
  // implementationsabhaegig ist, wenn a<0.
  int r=a % n;
  if (r<0) {
    r += n;
  }
#ifdef ASKOS_ERRORCHECK
  if ( r<0 || r>=n ) {
    FATAL_ERROR;
  }
#endif
  return r;
}


#ifndef __GNUC__ // Borland C++ hat kein lgamma.
double lgamma(const double r) {
  double x,tmp,ser;
  static double cof[6]={76.18009173,-86.50532033,24.01409822,
			-1.231739516,0.120858003e-2,-0.536382e-5};
  int j;
  x=r-1.0;
  tmp=x+5.5;
  tmp -= (x+0.5)*log(tmp);
  ser=1.0;
  for (j=0;j<=5;j++) {
    x += 1.0;
    ser += cof[j]/x;
  }
  return -tmp+log(2.50662827465*ser);
}  
#endif

