/***************************************************************************
                          askosmath.h  -  description
                             -------------------
    begin                : Thu Nov 28 2002
    author               : Moritz Franosch
    copyright            : (C) 2002 by ASKOS http://www.ASKOS.de
    email                : mail@Franosch.org
 ***************************************************************************/

#ifndef ASKOSMATH_H
#define ASKOSMATH_H


#include <cstdlib>
#include <cmath>

#ifdef __BORLANDC__
#include <math.h>
// Borland C++ braucht <math.h> fuer fabs().
#endif


const long double pi=3.14159265358979323846264338327;


inline double sign(const double x) {
  return x==0 ? 0 : (x>0 ? 1 : -1);
}


inline double sqr(const double& x) {
  return x*x;
}


inline double heaviside(const double& x) {
  return ( x>0.0 ? double(1.0) : double(0.0) );
}


inline double rectify(const double& x) {
  return ( x>0.0 ? x : 0 );
}


/** Gibt die Zahl der Bits zurueck, die in a und b uebereinstimmen, gegeben
 *  die Groesse scaling der Zahlen.
 *
 *  Ist scaling=0, so wird scaling=max(a,b) gesetzt.
 *
 *  Ref.: Ian Oliver, Klassische Programmiertechnik, Prentice Hall 1994.
 */
double similar_bits(const double a, const double b, double scaling);


/** True, falls a und b aehnlich (d.i. gleich bis auf Fehler) sind.
 *
 *  Ref.: Ian Oliver, Klassische Programmiertechnik, Prentice Hall 1994.
 */
bool similar(const double a, const double b, const double scaling);


/** Gibt a mod n zurueck mit 0 <= a mod n < n.
 *
 *  Voraussetzung: n>0.
 */
int remainder(const int a, const int n);


/** Gibt den natuerlichen Logarithmus der Gamma-Funktion ln(Gamma(x))
 *  zurueck.
 */
#ifndef __GNUC__ // Borland C++ hat kein lgamma.
double lgamma(const double x);
#endif


/** Gibt den Absolutwert eines doubles zurueck.
 */
#ifdef __BORLANDC__
  inline double abs(double x) {
    // Borland C++ hat kein double abs(double x) in <cmath> (!).
    return fabs(x);
  }
#endif


#endif // ASKOSMATH_H
