/***************************************************************************
                          askosrand.cpp  -  description
                             -------------------
    begin                : Tue Oct 22 2002
    author               : Moritz Franosch
    copyright            : (C) 2002 by ASKOS http://www.ASKOS.de
    email                : mail@Franosch.org
 ***************************************************************************/


#include <cstdlib>
#include <limits>

#include "askosrand.h"
#include "askosmath.h"


using namespace std;


Uniform_Rand::Uniform_Rand(const double rmin_ini, const double rmax_ini) : 
  rmin(rmin_ini), rmax(rmax_ini) {
}


double Uniform_Rand::operator()() {
  // rand() gibt im Bereich [0,RAND_MAX[ zurueck, deshalb ist der hier
  // zurueckgegebene Wert im Bereich [rmin,rmax[.
  return rmin+rmax*(double(rand())/double(RAND_MAX));
}


Gaussian_Rand::Gaussian_Rand(const double mean_ini, const double sigma_ini) :
  mean(mean_ini),
  sigma(sigma_ini),
  iset(false),
  gset(numeric_limits<double>::quiet_NaN()),
  uniform_rand(0,1) {
}


double Gaussian_Rand::operator()() {
  // siehe 
  // Press, William H.; Teukolsky, Saul A.; Vetterling, William T.; Flannery, Brian P.,
  // Numerical Recipes in C, Cambridge University Press, 1995
  if (!iset) {
    double v1, v2, rsq;
    do {
      v1=2*uniform_rand()-1;
      v2=2*uniform_rand()-1;
      rsq=sqr(v1)+sqr(v2);
    } while (rsq>=1 || rsq==0);
    double fac=sqrt(-2*log(rsq)/rsq);
    gset=sigma*v1*fac+mean;
    iset=true;
    return sigma*v2*fac+mean;
  } else {
    iset=false;
    return gset;
  }
}
