/***************************************************************************
                          askosrand.h  -  description
                             -------------------
    begin                : Tue Oct 22 2002
    author               : Moritz Franosch
    copyright            : (C) 2002 by ASKOS http://www.ASKOS.de
    email                : mail@Franosch.org
 ***************************************************************************/

#ifndef ASKOSRAND_H
#define ASKOSRAND_H


/** Basisklasse fuer Zufallszahlengeneratoren.
 */
class Rand {

public:

  /** Gibt die Zufallszahl zurueck.
   */
  virtual double operator()()=0;

};


/** Zufallszahlengenerator fuer gleichverteilte Zufallszahlen.
 */
class Uniform_Rand : public Rand {

private:

  double rmin, rmax; // Zufallszahlen werden aus [rmin,rmax[ gezogen.

public:

  /** Konstruiert einen Zufallszahlengenerator fuer gleichverteilte
   *  Zufallszahlen aus [rmin,rmax[.
   */
  Uniform_Rand(const double rmin, const double rmax);

  virtual double operator()();

};


/** Zufallszahlengenerator fuer normalverteilte Zufallszahlen.
 */
class Gaussian_Rand {

private:

  double mean, sigma; // Mittelwert und Standardabweichung
  bool iset; // normalverteilte Zufallszahl ist in gset gespeichert
  double gset; // gespeicherte normalverteilte Zufallszahl
  Uniform_Rand uniform_rand; // liefert gleichverteilte Zufallszahlen aus [0,1[.

public:

  /** Konstruiert einen Zufallszahlengenerator fuer normalverteilte
   *  Zufallszahlen mit Mittelwert mean und Standardabweichung sigma.
   */
  Gaussian_Rand(const double mean, const double sigma);

  virtual double operator()();

};


#endif // ASKOSRAND_H
