/***************************************************************************
                          statistics.h  -  description
                             -------------------
    begin                : Wed Nov 27 2002
    author               : Moritz Franosch
    copyright            : (C) 2002 by ASKOS http://www.ASKOS.de
    email                : mail@Franosch.org
 ***************************************************************************/


#ifndef STATISTICS_H
#define STATISTICS_H


#include <map>
#include <limits>


/** Kumulative Binomialverteilung
 *  sum(i=n bis N) binomial(N,n)*p^i*(1-p)^(N-i)
 */
double cumulative_binomial(const int N, const int n, const double p);


/** Incomplete beta function I_x(a,b).
 */
double incomplete_beta(const double a, const double b, double x);


/** Continued fraction fuer incomplete_beta().
 */
double beta_continued_fraction(const double a, const double b, const double x);


/** Normalverteilung mit Standardabweichung 1.
 *
 *  1/sqrt(2*pi) * exp(-0.5*z^2)
 */
double normal(const double z);


/** Kumulative Normalverteilung (Gausssches Fehlerintegral).
 *  int_-infty^x  normal(z) dz
 */
double cumulative_normal(const double z);


/** Incomplete gamma function P(a,x) = 1/Gamma(a) int_0^x exp(-t) t^(a-1) dt.
 */
double incomplete_gamma_P(const double a, const double x);


/** Incomplete gamma function Q(a,x) = 1-P(a,x).
 */
double incomplete_gamma_Q(const double a, const double x);


/** Wahrscheinlichkeit, dass eine mit n Freiheitsgraden
 *  Chi-Quadrat-verteilte Zufallsgroesse kleiner als chi_sqr ist.
 */
double prob_chi_square(const double chi_sqr, const double n);


/** Berechnung der Quantile der chi^2-Verteilung.
 */
class Quantile_chi_square {

private:

  // ordnet einer Wahrscheinlichkeit p und der Zahl der Freiheitsgrade
  // n das Quantil q zu, so dass P(chi^2<q)=p.
  mutable std::map< std::pair<double,int>, double> cache;

public:

  // Gibt zu einer Wahrscheinlichkeit p das Quantil q zurueck, so dass
  // fuer die Wahrscheinlichkeit P(chi^2<r), dass eine mit n
  // Freiheitsgraden chi^2-verteile Zufallsgroesse kleiner als q ist,
  // P(chi^2<r)=p gilt.
  //
  // Schon einmal berechnete Werte werden in einem Cache gespeichert.
  double operator()(const double p, const int n) const;
  
};


/** Gibt die "error function" erf(x)=2/sqrt(pi) int_0^x exp(-t^2) dt
 *  zurueck.
 */
#ifndef __GNUC__ // Borland C++ hat keine erf.
double erf(const double x);
#endif


#endif // STATISTICS_H
