/***************************************************************************
                          askosparam.cpp  -  description
                             -------------------
    begin                : Tue Apr 9 2002
    author               : Moritz Franosch
    copyright            : (C) 2002 by ASKOS http://www.ASKOS.de
    email                : mail@Franosch.org
 ***************************************************************************/

#include <fstream>
#include <deque>
#include <stack>

#include "askosdata/tree.h"
#include "askosio/log.h"
#include "askosio/string_convert.h"
#include "askosparam.h"


// Liest einen String aus istream& in.
// Die Zeichen in belongs_to_string goehoeren zu Strings,
// ansonsten nur alphanumerische Zeichen, "_" und ".".
bool read_string(istream& in, string& s, const string belongs_to_string="") {
  in >> ws;
  char c;
  if ( !in.get(c) ) {
    return false;
  }
  s="";
  if (c=='"') {
    while ( in.get(c) && c!='"' ) {
      s+=c;
    }
    if (c!='"') {
      return false;
    }
  } else {
    in.putback(c);
    while ( in.get(c) ) {
      if ( isalnum(c) || c=='_' || c=='.' || c=='-' || belongs_to_string.find(c)!=string::npos ) {
	s+=c;
      } else {
	in.putback(c);
	if (s=="") {
	  return false;
	}
	return true;
      }
    }
    if (s=="") {
      return false;
    }
  }
  return true;
}




Askosparam::Askosparam(Askosparam_Callback* callback_ptr_ini, 
		       Logdevice* logdevice_ptr_ini) : 
  callback_ptr(callback_ptr_ini), logdevice_ptr(logdevice_ptr_ini),
  types_ptr(new map<string,Tree_t>),
  types(*types_ptr),
  filename(""), 
  directory(".") {
}


Askosparam::Askosparam(Askosparam_Callback* callback_ptr_ini, 
		       Logdevice* logdevice_ptr_ini,
		       const string& directory_ini) : 
  callback_ptr(callback_ptr_ini), logdevice_ptr(logdevice_ptr_ini),
  types_ptr(new map<string,Tree_t>),
  types(*types_ptr),
  filename(""), 
  directory(directory_ini) {
}


Askosparam::Askosparam(Askosparam_Callback* callback_ptr_ini, 
		       Logdevice* logdevice_ptr_ini,
		       int argc,
		       char* argv[]) : 
  callback_ptr(callback_ptr_ini), logdevice_ptr(logdevice_ptr_ini),
  types_ptr(new map<string,Tree_t>),
  types(*types_ptr),
  filename("") {
  Log out(logdevice_ptr,"Askosparam","Askosparam");
  if (argc<=1) {
    out.error() << "No parameter files given on command line." << endl;
    out.error_exit();
  }
  directory=string_convert::dirname(argv[0]);
  // Eingabedateien auf der Kommandozeile einlesen.
  // Callback-Members in callback werden aufgerufen.
  for (int i=1; i<argc; i++) {
    out.info() << "Parsing parameter file '" << argv[i] << "'." << endl;
    if ( !parse(argv[i]) ) {
      out.error() << "Parsing parameter file '" << argv[i] << "' failed." << endl;
      out.error_exit();
    }
  }
}


bool Askosparam::parse(const string& filename_ini) {
  Log out(logdevice_ptr,"Askosparam","parse");
  filename=filename_ini;
  out.debug() << "filename '" << filename << "'" << endl;
  ifstream in(filename.c_str());
  if (!in) {
    out.error() << "Open file '" << filename << "' failed." << endl;
    out.error_exit();
  }
  return parse(in);
}


bool Askosparam::parse(istream& in) {
  Log out(logdevice_ptr,"Askosparam","parse");
  string name="";
  if (! (in >> name) || name!="ASKOSformat") {
    out.error() << "Parameter file '" << filename 
		<< "' must begin with 'ASKOSformat', read '" 
		<< name << "'." << endl;
    return false;
  }
  string formatfilename="";
  if (! (in >> formatfilename) ) {
    out.error() << "Format filename expected, got '" << "'." << endl;
    return false;
  }
  parse_formatfile(formatfilename);
  while (in >> name) {
    if ( name.size()>0 && name[0]=='#' ) {
      string line;
      getline(in,line);
      continue;
    }
    if (name=="debug") {
      in >> ws;
      char c;
      if ( !in.get(c) || c!='(' ) {
	out.error() << "'(' expected." << endl;
	return false;
      }
      for (;;) {
	in >> ws;
	if ( !in.get(c) ) {
	  out.error() << "Unexpected end of file." << endl;
	  return false;
	}
	if ( c!=')' ) {
	  in.putback(c);
	} else {
	  break;
	}
	string classname;
	in >> classname;
	logdevice_ptr->debug_on(classname);
      }
      continue;
    }
    out.debug() << "read parameter name '" << name << "'" << endl;
    callback_ptr->got_parameter_called_by_askosparam(name);
    map<string,Tree_t>::iterator types_it=types.find(name);
    if ( types_it==types.end() ) {
      out.error() << "No type defined for parameter '" << name << "'." << endl;
      return false;
    }
    Tree_t::iterator tree_it=types_it->second.begin();
    int level=0;
    bool is_list=false;
    stack<Tree_t::iterator> tree_it_stack;
    while ( tree_it!=types_it->second.end() ) {
      if ( tree_it.at_end() ) {
	in >> ws;
	char c;
	if ( !in.get(c) || c!=')' ) {
	  out.error() << "')' expected." << endl;
	  return false;
	}
	out.debug() << "Exit level " << level << "." << endl;
	--level;
	callback_ptr->list_end_called_by_askosparam(level);
      } else if ( tree_it->is_leaf() ) {
	string type_name=tree_it->get_leaf().get_name();
	if ( type_name=="float" ) {
	  double r;
	  if ( !(in >> r) ) {
	    out.error() << "Float expected." << endl;
	    return false;
	  }
	  out.debug() << "Got float " << r << "." << endl;
	  callback_ptr->got_float(r);
	} else if ( type_name=="string" ) {
	  string r;
	  if ( !read_string(in,r) ) {
	    out.error() << "String expected." << endl;
	    return false;
	  }
	  out.debug() << "Got string " << r << "." << endl;
	  callback_ptr->got_string(r);
	} else if ( type_name=="[string]" ) {
	  string r;
	  if ( read_string(in,r) ) {
	    out.debug() << "Got [string] " << r << "." << endl;
	    callback_ptr->got_string(r);
	  } else {
	    out.debug() << "Got no [string]." << endl;
	  }
	} else if ( type_name=="int" ) {
	  int r;
	  if ( !(in >> r) ) {
	    out.error() << "Integer expected." << endl;
	    return false;
	  }
	  out.debug() << "Got integer " << r << "." << endl;
	  callback_ptr->got_int(r);
	} else if ( type_name=="bool" ) {
	  string s;
	  if ( !(in >> s) ) {
	    out.error() << "Boolean expected." << endl;
	    return false;
	  }
	  bool b;
	  if (s=="yes" || s=="true") {
	    b=true;
	  } else if (s=="no" || s=="false") {
	    b=false;
	  } else {
	    out.error() << "Boolean expected. Got '" << s << "'." << endl;
	    return false;
	  }
	  out.debug() << "Got boolean '" << s << "'." << endl;
	  callback_ptr->got_bool(b);
	} else if ( type_name=="list" ) {
	  is_list=true;
	} else {
	  out.error() << "Unknown type name '" << type_name << "'." << endl;
	  return false;
	}
      } else {
	in >> ws;
	char c;
	if ( !in.get(c) || c!='(' ) {
	  out.error() << "'(' expected." << endl;
	  return false;
	}
	out.debug() << "Enter level " << level+1 << "." << endl;
	callback_ptr->list_begin_called_by_askosparam(level);
	++level;
      }
      if (is_list) {
	out.debug() << "Got list." << endl;
	is_list=false;
	tree_it.step();
	tree_it_stack.push(tree_it);
      } else {
	if ( tree_it_stack.empty() ) {
	  tree_it.step();
	} else {
	  out.debug() << "Stack not empty." << endl;
	  bool gone_up=false;
	  if ( tree_it.at_end() ) {
	    out.debug() << "Go up." << endl;
	    tree_it.up();
	    gone_up=true;
	  }
	  if ( tree_it_stack.top()!=tree_it ) {
	    out.debug() << "Next element." << endl;
	    if (gone_up) {
	      tree_it.next();
	    } else {
	      tree_it.step();
	    }
	  } else {
	    in >> ws;
	    char c;
	    if ( !in.get(c) || c!=')' ) {
	      // list continues
	      out.debug() << "List continues." << endl;
	      if ( !tree_it->is_leaf() && !gone_up ) {
		tree_it.step();
	      }
	    } else {
	      // end of list
	      out.debug() << "End of list." << endl;
	      tree_it_stack.pop();
	      tree_it.next();
	    }
	    in.putback(c);
	  }
	}
      }
    }
  }
  callback_ptr->finished();
  return true;
}


bool Askosparam::parse_formatfile(const string& formatfilename) {
  Log out(logdevice_ptr,"Askosparam","parse_formatfile");
  string full_formatfilename=directory+"/"+formatfilename;
  ifstream in(full_formatfilename.c_str());
  if (!in) {
    out.error() << "Open format file '" << full_formatfilename << "' failed." << endl;
    out.error_exit();
  }
  string name;
  while (in >> name) {
    if ( name.size()>0 && name[0]=='#' ) {
      string line;
      getline(in,line);
      continue;
    }
    out.debug() << "read parameter name '" << name << "'" << endl;
    if ( types.find(name)!=types.end() ) {
      out.error() << "Type for parameter '" << name << "' already defined." << endl;
      return false;
    }
    Tree_t& new_tree(types[name]);
    Tree_t::iterator tree_it=new_tree.begin();
    int level=0;
    do {
      in >> ws;
      char c;
      if ( !in.get(c) ) {
	out.error() << "Character expected." << endl;
	return false;
      }
      if (c=='(') {
	level++;
	tree_it=tree_it.insert_tree();
	tree_it.step(); // step into the newly inserted tree
      } else if (c==')') {
	out.debug() << "Got ')'." << endl;
	if (level==0) {
	  out.error() << "Got ')', but already at level 0." << endl;
	  return false;
	}
	level--;
	tree_it.step(); // go up to the parent
      } else {
	in.putback(c);
	string type_name;
	if ( !read_string(in,type_name,"[]") ) {
	  out.error() << "Typename expected." << endl;
	  return false;
	}
	out.debug() << "got typename '" << type_name << "'" << endl;
	tree_it.insert(Type(type_name)); // insert a leaf
      }
    } while ( level>0 );
    out.debug() << "Read type tree '" << types[name] << "'." << endl;
  }
  return true;
}
