/***************************************************************************
                          askosparam_callback.cpp  -  description
                             -------------------
    begin                : Tue Apr 9 2002
    author               : Moritz Franosch
    copyright            : (C) 2002 by ASKOS http://www.ASKOS.de
    email                : mail@Franosch.org
 ***************************************************************************/


#include "askosio/errorcheck.h"

#include "askosparam_callback.h"


void Askosparam_Callback::got_parameter_called_by_askosparam(const string& name) {
#ifdef ASKOS_ERRORCHECK
  if ( nvariables.size()==0 ) {
    FATAL_ERROR;
  }
#endif
  last_parameter=name;
  ++nvariables.top();
  got_parameter(name);
}


void Askosparam_Callback::list_begin_called_by_askosparam(const int level) {
#ifdef ASKOS_ERRORCHECK
  if ( nvariables.size()==0 ) {
    FATAL_ERROR;
  }
#endif
  actual_level=level;
  ++nvariables.top();
  nvariables.push(0);
  list_begin(level);
}


void Askosparam_Callback::list_end_called_by_askosparam(const int level) {
#ifdef ASKOS_ERRORCHECK
  if ( nvariables.size()==0 ) {
    FATAL_ERROR;
  }
#endif
  actual_level=level;
  nvariables.pop();
  list_end(level);
}


Askosparam_Callback::Askosparam_Callback() : 
  last_parameter(),
  floats(),
  ints(),
  strings(),
  bools(),
  actual_level(0),
  nvariables() {
  nvariables.push(0);
}


Askosparam_Callback::~Askosparam_Callback() {
}


void Askosparam_Callback::insert_string(const string& r) {
  if ( strings.find(get_last_parameter())!=strings.end() ) {
    FATAL_ERROR_TEXT("String '"+get_last_parameter()+"' already exists.");
  }
  strings[get_last_parameter()]=r;
}


void Askosparam_Callback::insert_float(const double r) {
  if ( floats.find(get_last_parameter())!=floats.end() ) {
    FATAL_ERROR_TEXT("Float '"+get_last_parameter()+"' already exists.");
  }
  floats[get_last_parameter()]=r;
}


void Askosparam_Callback::insert_int(const int r) {
  if ( ints.find(get_last_parameter())!=ints.end() ) {
    FATAL_ERROR_TEXT("Integer '"+get_last_parameter()+"' already exists.");
  }
  ints[get_last_parameter()]=r;
}


void Askosparam_Callback::insert_bool(const bool r) {
  if ( bools.find(get_last_parameter())!=bools.end() ) {
    FATAL_ERROR_TEXT("Bool '"+get_last_parameter()+"' already exists.");
  }
  bools[get_last_parameter()]=r;
}


double Askosparam_Callback::get_float(const string& r) {
  if ( floats.find(r)==floats.end() ) {
    FATAL_ERROR_TEXT("Float '"+r+"' does not exist.");
  }
  return floats[r];
}


int Askosparam_Callback::get_int(const string& r) {
  if ( ints.find(r)==ints.end() ) {
    FATAL_ERROR_TEXT("Integer '"+r+"' does not exist.");
  }
  return ints[r];
}


const string& Askosparam_Callback::get_string(const string& r) {
  if ( strings.find(r)==strings.end() ) {
    FATAL_ERROR_TEXT("String '"+r+"' does not exist.");
  }
  return strings[r];
}


bool Askosparam_Callback::get_bool(const string& r) {
  if ( bools.find(r)==bools.end() ) {
    FATAL_ERROR_TEXT("Bool '"+r+"' does not exist.");
  }
  return bools[r];
}


int Askosparam_Callback::get_number_of_variable_in_level() const {
#ifdef ASKOS_ERRORCHECK
  if ( nvariables.size()==0 ) {
    FATAL_ERROR;
  }
#endif
  return nvariables.top();
}


void Askosparam_Callback::got_parameter(const string&) {
}


void Askosparam_Callback::got_float(const double r) {
  insert_float(r);
}


void Askosparam_Callback::got_int(const int r) {
  insert_int(r);
}


void Askosparam_Callback::got_string(const string& r) {
  insert_string(r);
}


void Askosparam_Callback::got_bool(const bool r) {
  insert_bool(r);
}


void Askosparam_Callback::list_begin(const int) {
}


void Askosparam_Callback::list_end(const int) {
}


void Askosparam_Callback::finished() {
}
