/***************************************************************************
                          askosparam_callback.h  -  description
                             -------------------
    begin                : Tue Apr 9 2002
    author               : Moritz Franosch
    copyright            : (C) 2002 by ASKOS http://www.ASKOS.de
    email                : mail@Franosch.org
 ***************************************************************************/

#ifndef ASKOSPARAM_CALLBACK_H
#define ASKOSPARAM_CALLBACK_H


#include <string>
#include <map>
#include <stack>


using namespace std;


class Askosparam_Callback {

private:

  string last_parameter;
  map<string,double> floats;
  map<string,int> ints;
  map<string,string> strings;
  map<string,bool> bools;

  // Aktuelle Klammerebene, wie zuletzt von list_begin und list_end zurueckgegeben.
  int actual_level;

  // Nummer der Variablen in der der jeweiligen Klammerebene.
  stack<int> nvariables;

  friend class Askosparam;

  void got_parameter_called_by_askosparam(const string& name);

  void list_begin_called_by_askosparam(const int level);

  void list_end_called_by_askosparam(const int level);

public:

  Askosparam_Callback();

  virtual ~Askosparam_Callback();

  void insert_float(const double r);

  void insert_int(const int r);
  
  void insert_string(const string& r);

  void insert_bool(const bool r);

  bool has_float(const string& r) const {
    return ( floats.find(r)!=floats.end() );
  }

  bool has_int(const string& r) const {
    return ( ints.find(r)!=ints.end() );
  }

  bool has_string(const string& r) const {
    return ( strings.find(r)!=strings.end() );
  }

  double get_float(const string& r);

  int get_int(const string& r);

  const string& get_string(const string& r);

  bool get_bool(const string& r);

  const string& get_last_parameter() const {
    return last_parameter;
  }

  int get_level() const {
    return actual_level;
  }

  int get_number_of_variable_in_level() const;

  virtual void got_parameter(const string& name);

  virtual void got_float(const double r);

  virtual void got_int(const int r);

  virtual void got_string(const string& r);

  virtual void got_bool(const bool r);

  // Wird beim Beginn einer Liste aufgerufen.
  // level enthaelt die Klammerebene
  //        (  ()  )
  // Ebene  ^0 ^1
  virtual void list_begin(const int level);

  // Wird beim Ende einer Liste ausgerufen.
  // level enthaelt die Klammerebene
  //        (  ()  )
  // Ebene      ^1 ^0
  virtual void list_end(const int level);

  virtual void finished();

};


#endif // ASKOSPARAM_CALLBACK_H
