/***************************************************************************
                          simple_parameterfile.cpp  -  description
                             -------------------
    begin                : Wed Jan 28 2004
    author               : Moritz Franosch
    copyright            : (C) 2004 by ASKOS http://www.ASKOS.de
    email                : mail@Franosch.org
 ***************************************************************************/

#include <fstream>

#include "askosio/log.h"
#include "askosio/string_convert.h"
#include "askosdates/convert_date.h"

#include "simple_parameterfile.h"


using namespace std;


Simple_Parameterfile::Simple_Parameterfile() {
}


Simple_Parameterfile::~Simple_Parameterfile() {
}


bool Simple_Parameterfile::parse(istream& input) {
  LOG(log);
  string parameter_name, value;
  while (input >> parameter_name) {
    if ( parameter_name=="#" ) {
      // Kommentarzeile ueberspringen
      string s;
      // getline(input,s); Loest bei Borland C++ einen Segfault aus.
      // char a[1000];
      // input.getline(a,999); Auch Segfault.
      char c=' ';
      while ( c!='\n' ) {
	input.get(c);
      }
      continue;
    }

    // Wir akzeptieren einen Parameterwert in Anfuehrungszeichen "".
    input >> ws;
    char c=' ';
    input.get(c);
    if ( c=='"' ) {
      value="";
      input.get(c);
      while ( c!='\n' && c!='"' ) {
	value += c;
	input.get(c);
      }
      if ( c=='\n' ) {
	log.error() << "Value of parameter '" << parameter_name 
		    << "' begins with '\"', but does not end before end of line." << endl;
	return false;
      }
    } else {
      input.putback(c);
      if ( ! (input >> value) ) {
	log.error() << "Value of parameter '" <<  parameter_name << "' is missing." << endl;
	return false;
      }
    }

    if (parameter_name=="output") {
      // Die Ausgabe soll hierin dupliziert werden.
      string logfile_name=get_string("logdir")+value+"-"+time_to_timestamp(time(0))+".log";
      Pointer<ostream> logfile_ptr(new ofstream(logfile_name.c_str()));
      if ( ! (*logfile_ptr) ) {
	log.error() << "Could not open logfile '" << logfile_name << "'." << endl;
      } else {
	log.get_default_logdevice().add_stream(logfile_ptr);
	log.info() << "We start logging into file '" << logfile_name << "'." << endl;
      }
    }

    log.info() << "Parameter " << parameter_name << "=" << value << "." << endl;
    double r=0;
    if ( string_convert::string_to_double(value,r) ) {
      // Konvertierung hat geklappt
      numbers[parameter_name]=Sdouble(r);
    } else {
      // Konvertierung hat nicht, geklappt Wert ist ein String
      strings[parameter_name]=value;
    }
  }
  return true;
}


void Simple_Parameterfile::missing_parameter(const string& message) const {
  LOG(log);
  log.warn() << message << endl;
}


const string Simple_Parameterfile::get_string(const string& parameter_name) {
  if ( strings[parameter_name]=="" ) {
    missing_parameter("No parameter '"+parameter_name+"' with string value specified.");
  }
  return strings[parameter_name];
}


Sdouble Simple_Parameterfile::get_number(const string& parameter_name) {
  if ( ! numbers[parameter_name].is_valid() ) {
    missing_parameter("No parameter '"+parameter_name+"' with numeric value specified.");
  }
  return numbers[parameter_name];
}


double Simple_Parameterfile::get_double(const std::string& parameter_name) {
  return get_number(parameter_name).to_double();
}


double Simple_Parameterfile::get_double(const std::string& parameter_name, const double default_value) {
  if ( !has_number(parameter_name) ) {
    return default_value;
  }
  return get_double(parameter_name);
}


bool Simple_Parameterfile::has_number(const string& parameter_name) {
  return numbers[parameter_name].is_valid();
}


bool Simple_Parameterfile::has_string(const string& parameter_name) {
  return strings[parameter_name]!="";
}
