/***************************************************************************
                          simple_parameterfile.h  -  description
                             -------------------
    begin                : Wed Jan 28 2004
    author               : Moritz Franosch
    copyright            : (C) 2004 by ASKOS http://www.ASKOS.de
    email                : mail@Franosch.org
 ***************************************************************************/

#ifndef SIMPLE_PARAMETERFILE_H
#define SIMPLE_PARAMETERFILE_H

#include <iostream>
#include <string>
#include <map>

#include "askosdata/sdouble.h"


/** Liest Parameter aus einem istream.
 *
 *  Die Datei bzw. der Stream hat das Format
 *  Parameter1 Wert1
 *  Parameter2 Wert2
 */
class Simple_Parameterfile {

private:

  std::map<std::string,std::string> strings; // Parameter, die Strings sind (Name, Wert)
  std::map<std::string,Sdouble> numbers; // Parameter, die Zahlen sind (Name, Wert)

public: 

  /** Konstruktor.
   */
  Simple_Parameterfile();

  /** Virtueller Destruktor.
   */
  virtual ~Simple_Parameterfile();

  /** Startet das Parsen des Parameterfiles.
   */
  bool parse(std::istream& input);

  /** Wird aufgerufen, wenn ein Parameter nicht vorhanden ist.
   *
   *  Gibt eine die Warnung 'message' aus.
   */
  virtual void missing_parameter(const std::string& message) const;

  /** Gibt den Wert des Parameters zurueck, wenn er als string
   *  interpretiert wurde.
   *
   *  Gibt "" zurueck, falls kein solcher Parameter gefunden wurde.
   */
  const std::string get_string(const std::string& parameter_name);

  /** Gibt den Wert des Parameters zurueck, wenn er als Zahl
   *  interpretiert wurde.
   *
   *  Gibt Sdouble::undefined zurueck, falls kein solcher Parameter
   *  gefunden wurde.
   */
  Sdouble get_number(const std::string& parameter_name);

  /** Gibt den Wert des Parameters zurueck, wenn er als Zahl
   *  interpretiert wurde.
   *
   *  Gibt NaN zurueck, falls kein solcher Parameter gefunden wurde.
   */
  double get_double(const std::string& parameter_name);

  /** Gibt den Wert des Parameters zurueck, wenn er als Zahl
   *  interpretiert wurde.
   *
   *  Gibt default_value zurueck, falls kein solcher Parameter
   *  gefunden wurde.
   */
  double get_double(const std::string& parameter_name, const double default_value);

  /** Existiert ein solcher Parameter?
   */
  bool has_number(const std::string& parameter_name);

  /** Existiert ein solcher Parameter?
   */
  bool has_string(const std::string& parameter_name);

};


#endif // SIMPLE_PARAMETERFILE_H
