/***************************************************************************
                          testmath.cpp  -  description
                             -------------------
    begin                : Wed Jan 15 2003
    author               : Moritz Franosch
    copyright            : (C) 2003 by ASKOS http://www.ASKOS.de
    email                : mail@Franosch.org
 ***************************************************************************/

#include <iostream>

#include "askosmath/statistics.h"
#include "askosmath/askosmath.h"

#include "testmath.h"


using namespace std;


double myerf(const double x) {
  return x<0 ? -incomplete_gamma_P(0.5,sqr(x)) : incomplete_gamma_P(0.5,sqr(x));
}


int TestMath::operator()() {

  // cumulative_normal

  if ( ! similar(cumulative_normal(0),0.5,1) ) {
    return EXIT_FAILURE;
  }

  if ( ! similar(cumulative_normal(1000000),1,1) ) {
    return EXIT_FAILURE;
  }

  if ( ! similar(cumulative_normal(-1000000),0,1) ) {
    return EXIT_FAILURE;
  }

  double e=0.0001;

  if ( abs(cumulative_normal(0.01)-0.5040)>e ) {
    return EXIT_FAILURE;
  }

  if ( abs(cumulative_normal(0.5)-0.6915)>e ) {
    return EXIT_FAILURE;
  }

  if ( abs(cumulative_normal(1)-0.8413)>e ) {
    return EXIT_FAILURE;
  }

  if ( abs(cumulative_normal(3)-0.9987)>e ) {
    return EXIT_FAILURE;
  }

  if ( !similar(cumulative_normal(0.01)+cumulative_normal(-0.01),1,1) ) {
    return EXIT_FAILURE;
  }

  if ( !similar(cumulative_normal(0.5)+cumulative_normal(-0.5),1,1) ) {
    return EXIT_FAILURE;
  }


  // cumulative_binomial

  cerr << "B(0,0,0.1)=" << cumulative_binomial(0,0,0.1) << endl;
  cerr << "B(1,0,0.1)=" << cumulative_binomial(1,0,0.1) << endl;
  cerr << "B(1,1,0.1)=" << cumulative_binomial(1,1,0.1) << endl;
  cerr << "B(2,0,0.1)=" << cumulative_binomial(2,0,0.1) << endl;
  cerr << "B(2,1,0.1)=" << cumulative_binomial(2,1,0.1) << endl;
  cerr << "B(2,2,0.1)=" << cumulative_binomial(2,2,0.1) << endl;
  cerr << "B(3,0,0.1)=" << cumulative_binomial(3,0,0.1) << endl;
  cerr << "B(3,1,0.1)=" << cumulative_binomial(3,1,0.1) << endl;
  cerr << "B(3,2,0.1)=" << cumulative_binomial(3,2,0.1) << endl;
  cerr << "B(3,3,0.1)=" << cumulative_binomial(3,3,0.1) << endl;
  cerr << "B(4,0,0.1)=" << cumulative_binomial(4,0,0.1) << endl;
  cerr << "B(4,1,0.1)=" << cumulative_binomial(4,1,0.1) << endl;
  cerr << "B(4,2,0.1)=" << cumulative_binomial(4,2,0.1) << endl;
  cerr << "B(4,3,0.1)=" << cumulative_binomial(4,3,0.1) << endl;
  cerr << "B(4,4,0.1)=" << cumulative_binomial(4,4,0.1) << endl;
  cerr << "B(4,-10,0.1)=" << cumulative_binomial(4,-10,0.1) << endl;
  cerr << "B(4,-1,0.1)=" << cumulative_binomial(4,-1,0.1) << endl;
  cerr << "B(4,5,0.1)=" << cumulative_binomial(4,5,0.1) << endl;
  cerr << "B(4,10,0.1)=" << cumulative_binomial(4,10,0.1) << endl;

  // chi square

  Quantile_chi_square chi_sqr;

  e=0.1;

  if ( abs(chi_sqr(0.975,1)-5.023)>e ) {
    return EXIT_FAILURE;
  }

  if ( abs(chi_sqr(0.975,100)-129.6)>e ) {
    return EXIT_FAILURE;
  }

  if ( abs(chi_sqr(0.01,1)-0.00016)>e ) {
    return EXIT_FAILURE;
  }

  if ( abs(chi_sqr(0.01,10)-2.56)>e ) {
    return EXIT_FAILURE;
  }

  if ( abs(chi_sqr(0.01,100)-70.1)>e ) {
    return EXIT_FAILURE;
  }

  // erf

  cerr << "erf(0)=" << myerf(0) << endl;
  cerr << "erf(-1)=" << myerf(-1) << endl;
  cerr << "erf(1)=" << myerf(1) << endl;
  cerr << "erf(-2)=" << myerf(-2) << endl;
  cerr << "erf(2)=" << myerf(2) << endl;
  cerr << "erf(0.01)=" << myerf(0.01) << endl;
  cerr << "erf(1E-300)=" << myerf(1E-300) << endl;
  cerr << "erf(100)=" << myerf(100) << endl;
  cerr << "erf(1E300)=" << myerf(1E300) << endl;


  return EXIT_SUCCESS;
}
