#!/usr/bin/perl -w


use Getopt::Long;
use File::Basename;


if ( !GetOptions('width=i',\$width,
		 'height=i',\$height,
		 'povray-options=s',\$povray_options,
		 'povfile=s',\$povfile,
		 'outputdir=s',\$outputdir,
		 'help',\$help) ||
     defined($help) || !defined($povfile)
   ) {
  print STDERR "usage:\n";
  print STDERR "$0 [ --width=INTEGER ]  [ --height=INTEGER ] [ --povray-options=OPTIONS ] --povfile=FILE.pov [ --outputdir=OUTPUTDIR ] file1.xml file2.xml ...\n";
  exit 1;
}

if ( !defined($width) ) {
  $width=320;
}

if ( !defined($height) ) {
  $height=200;
}

$mydir = dirname($0);

$filelist="filelist.txt";

if ( !open(FILELIST,"> $filelist") ) {
  print STDERR "Could not open file $filelist for writing.";
  exit 1;
}

$file = shift;
while ( defined($file) ) {
  ($name, $dir, ) = fileparse($file,"\\..*");
  if ( defined($outputdir) ) {
    $dir=$outputdir;
  }
  if ( $dir eq "" ) {
    $dir = ".";
  }
  $name = $name.".png";
  $outfile = $dir."/".$name;
  print "Raytracing file $file to $outfile.\n";
  print `$mydir/xml_convert.pl --pov $file > elements.pov`;
  print `povray +I$povfile +O$outfile $povray_options +W$width +H$height`;
  print FILELIST "$name\n";
  $file = shift;
}
