#!/usr/bin/perl -w


use Getopt::Long;
use XML::Parser;


if ( !GetOptions('xmgrace',\$xmgrace,
		 'symbol-color=i',\$symbol_color,'symbol-size=f',\$symbol_size,
		 'split',\$split,
		 'prefix=s',\$prefix,'start=f',\$start,'end=f',\$end,
		 'pov',\$pov,
		 'depth=f',\$depth,'radius=f',\$radius,
		 'help',\$help) ||
     defined($help) ||
     ! ( defined($xmgrace) xor defined($split) xor defined($pov) )
   ) {
  print STDERR "usage:\n";
  print STDERR "$0 --xmgrace [ --symbol-color=INTEGER ]  [ --symbol-size=FLOAT ] [file.xml] > outputfile.xml\n";
  print STDERR "$0 --split [--prefix=STRING ] [ --start=FLOAT ]  [ --end=FLOAT ] [file.xml]\n";
  print STDERR "$0 --pov [ --depth=FLOAT ]  [ --radius=FLOAT ] [file.xml] > outputfile.pov\n";
  exit 1;
}

$default_text_handler = 0;
$default_start_tag_handler = 0;
$default_end_tag_handler = 0;

if ( defined($xmgrace) ) {
  $modus="xmgrace";
  if ( !defined($symbol_color) ) {
    $symbol_color = 4; # blue
  }
  if ( !defined($symbol_size) ) {
    $symbol_size = 0.25;
  }
  $frame_number=0;
}

if ( defined($split) ) {
  $modus="split";
  $split_number=0;
  if ( !defined($prefix) ) {
    $prefix = "";
  }
  if ( !defined($start) ) {
    $start = 0;
  }
}

if ( defined($pov) ) {
  $modus="pov";
  if ( !defined($depth) ) {
    $depth = 2;
  }
  if ( !defined($radius) ) {
    $radius = 0.1;
  }
}

$file = shift;

if ( !defined($file) ) {
  $file="-";
}

$parser = new XML::Parser(Style => 'Stream', ErrorContext => 2);

$text_handler = $default_text_handler;
$parser -> parsefile($file);

exit 0;

# end of main




# Subroutines used by several $modus.


sub print_text_handler {
  print $_;
}


sub print_tag_handler {
  print $_;
}


sub get_coordinates {
  (my $x, my $y, my $z) = split ' ',$_;
  #if ( defined($pov) ) {
  #  $y=-$y; # povray uses a left-handed coordinate system
  #}
  return ($x, $y, $z);
}



# These subroutines are called by $parser->parsefile().

# They in turn call the subroutines (handlers) "$modus_TAG" on start
# of tag TAG, "$modus_TAG_" on end of tag TAG and "$modus_text" for
# text blocks between <TAG> and </TAG>.

sub Text {
  if ( defined(&$text_handler) ) {
    &$text_handler;
  }
  $text_handler = $default_text_handler;
}


sub StartTag {
  shift;
  $tag = shift;
#  print "[start of $tag]";
  if ( defined(&{$modus."_".$tag}) ) {
    &{$modus."_".$tag};
  } else {
    if ( defined(&$default_start_tag_handler) ) {
      &$default_start_tag_handler;
    }
  }
  if ( defined(&{$modus."_".$tag."_text"}) ) {
    $text_handler = \&{$modus."_".$tag."_text"};
  } else {
    $text_handler = $default_text_handler;
  }
}


sub EndTag {
  shift;
  $tag = shift;
#  print "[end of $tag]";
  if ( defined(&{$modus."_".$tag."_"}) ) {
    &{$modus."_".$tag."_"};
  } else {
    if ( defined(&$default_end_tag_handler) ) {
      &$default_end_tag_handler;
    }
  }
}




# Handlers for $modus=xmgrace.


$in_space=0;


sub xmgrace_position {
  if ($in_space) {
    print "g0.s0 point ";
  }
}


sub xmgrace_position_ {
  if ($in_space) {
    print "\n";
  }
}


sub xmgrace_position_text {
  if ($in_space) {
    (my $x, my $y, my $z) = get_coordinates($_);
    if ($dimensions==2) {
      print $x,", ",$y;
    } elsif ($dimensions==3) {
      print $x,", ",$z;
    }
  }
}


sub xmgrace_output {
  print "\n";
  print "\n";
  print "kill g1.s0\n";
  print "kill g1.s1\n";
  print "kill g1.s2\n";
  print "kill g1.s3\n";
  print "g1.s0 line color 1\n";
  print "g1.s1 line color 2\n";
  print "g1.s2 line color 15\n";
  print "g1.s3 line color 4\n";

  print "g1.s0 line linewidth 4\n";
  print "g1.s1 line linewidth 4\n";
  print "g1.s2 line linewidth 4\n";
  print "g1.s3 line linewidth 4\n";

  print "kill g2.s0\n";
  print "g2.s0 line color 2\n";
  print "g2.s0 line linewidth 4\n";

  print "kill g3.s0\n";
  print "g3.s0 line color 4\n";
  print "g3.s0 line linewidth 4\n";
}


sub xmgrace_space {
  $in_space=1;
  my $t=$_{"time"};
  my $E=$_{"energy"};
  my $E_g=$_{"gravity_energy"};
  my $E_p=$_{"pressure_energy"};
  my $E_kin=$_{"kinetic_energy"};
  my $c_a=$_{"crit_acceleration"};
  my $rho_dev=$_{"density_deviation"};
  $dimensions=$_{"dimensions"};
  my $rho_dev_percent=100*$rho_dev;

  print STDERR "time ".$t." s \n";
  print STDERR "  energy ".$E." J \n";
  print STDERR "  gravity energy ".$E_g." J \n";
  print STDERR "  pressure energy ".$E_p." J \n";
  print STDERR "  kinetic energy ".$E_kin." J \n";
  print STDERR "  crit. acceleration ".$c_a."\n";
  print STDERR "  density_deviation ".$rho_dev_percent." %\n";

  print "\n";

  print "kill g0.s0\n";
  print "g0.s0 symbol 1\n";
  print "g0.s0 symbol size $symbol_size\n";
  print "g0.s0 symbol color $symbol_color\n";
  print "g0.s0 symbol pattern 1\n";
  print "g0.s0 symbol fill color 4\n";
  print "g0.s0 symbol fill pattern 1\n";
  print "g0.s0 linestyle 0\n";

  print "g1.s0 point $t, $E\n";
  print "g1.s1 point $t, $E_g\n";
  print "g1.s2 point $t, $E_p\n";
  print "g1.s3 point $t, $E_kin\n";

  print "g2.s0 point $t, $c_a\n";

  print "g3.s0 point $t, $rho_dev\n";

  print "with g4\n";
  print "    subtitle \"t = $t s\\n";
  print "E = $E J\\n";
  print "E\\sg\\N = $E_g J\\n";
  print "E\\sp\\N = $E_p J\\n";
  print "E\\skin\\N = $E_kin J\\n";
  print "r\\s\\f{Symbol}r\\f{}\\N = $rho_dev_percent %\\n\"\n";

}


sub xmgrace_space_ {
  print "redraw\n";

#  print "HARDCOPY DEVICE \"JPEG\"\n";
#  print "DEVICE \"JPEG\" PAGE SIZE $pixelwidth, $pixelheight\n";
#  print "DEVICE \"JPEG\" OP \"quality:85\"\n";
#  print "PRINT TO \"$framesdir/$framename.jpg\"\n";
#  print "PRINT\n";
#  print "HARDCOPY DEVICE \"EPS\"\n"; # JPEG,PNG erroneous
#  print "PRINT TO \"$framesdir/$framename.eps\"\n";
#  print "PRINT\n";
  if ( defined($prefix) ) {
    # tell xmgrace to put out the frame
    my $digits_number = sprintf("%06d", $frame_number);
    my $filename = $prefix.$digits_number.".png";
    print "DEVICE \"PNG\" PAGE SIZE 640, 480\n";
    print "HARDCOPY DEVICE \"PNG\"\n";
    print "PRINT TO \"$filename\"\n";
    print "PRINT\n";
  }

  ++$frame_number;
  $in_space=0;
}




# Handlers for $modus=split.

sub split_space {
  my $time = ${%_}{"time"};
  if ( $time>=$start && (!defined($end) || $time<=$end) ) {
    my $digits_number = sprintf("%06d", $split_number);
    my $filename = $prefix.$digits_number.".xml";
    print STDERR "Split number $split_number (time $time s) written to file '$filename'.\n";
    if ( ! open(SPLITFILE,"> $filename") ) {
      print STDERR "Could not open file '$filename' for writing.";
      exit 1;
    }
    ++$split_number;
    print SPLITFILE "<frame>\n";
    print SPLITFILE $_;
    $default_start_tag_handler = \&print_to_splitfile;
    $default_end_tag_handler = \&print_to_splitfile;
    $default_text_handler = \&print_to_splitfile;
  } else {
    if ( defined($end) && $time>$end ) {
      exit 0;
    }
    $default_start_tag_handler = 0;
    $default_end_tag_handler = 0;
    $default_text_handler = 0;
  }
}


sub split_spheres_ {
  print SPLITFILE $_;
  print SPLITFILE "\n</frame>\n";
  $default_start_tag_handler = 0;
  $default_end_tag_handler = 0;
  $default_text_handler = 0;
}


sub split_voxels_ {
  print SPLITFILE $_;
  print SPLITFILE "\n</frame>\n";
  $default_start_tag_handler = 0;
  $default_end_tag_handler = 0;
  $default_text_handler = 0;
}


# Subroutines for $modus=split.

sub print_to_splitfile {
  print SPLITFILE $_;
}




# Handlers for $modus=pov.


$in_spheres=0;


sub pov_simulation {
  print STDERR "Modus pov has no support for creating more than one .pov-Outputfile.\n";
  print STDERR "Please split into several frames with --split first.\n";
  exit 1;
}


sub pov_space {
  $dimensions=$_{"dimensions"};
}


sub pov_spheres {
  $in_spheres=1;
}


sub pov_spheres_ {
  $in_spheres=0;
}


sub pov_voxels {
  $in_voxels=1;
  # fixme
  print "mesh {\n";
}


sub pov_voxels_ {
  $in_voxels=0;
  # fixme
  print "texture { Water }\n";
  print "}\n";
}


sub pov_position_text {
  if ( $in_spheres ) {
    ($x, $y, $z) = get_coordinates($_);
  }
}


sub pov_radius_text {
  if ( $in_spheres ) {
    $radius = $_;
  }
}


sub pov_corner1_text {
  if ( $in_voxels ) {
    ($x, $y, $z) = get_coordinates($_);
  }
}


sub pov_corner2_text {
  if ( $in_voxels ) {
    ($x2, $y2, $z2) = get_coordinates($_);
  }
}


sub pov_corner3_text {
  if ( $in_voxels ) {
    ($x3, $y3, $z3) = get_coordinates($_);
  }
}


sub pov_normal1_text {
  if ( $in_voxels ) {
    ($n1x, $n1y, $n1z) = get_coordinates($_);
  }
}


sub pov_normal2_text {
  if ( $in_voxels ) {
    ($n2x, $n2y, $n2z) = get_coordinates($_);
  }
}


sub pov_normal3_text {
  if ( $in_voxels ) {
    ($n3x, $n3y, $n3z) = get_coordinates($_);
  }
}


sub pov_triangle_ {
  if ( $in_voxels ) {
    print "triangle { <$x,$z,$y>, <$x2,$z2,$y2>, <$x3,$z3,$y3> }\n";
  }
}


sub pov_smooth_triangle_ {
  if ( $in_voxels ) {
    print "smooth_triangle { <$x,$z,$y>, <$n1x,$n1z,$n1y>, <$x2,$z2,$y2>, <$n2x,$n2z,$n2y>, <$x3,$z3,$y3>, <$n3x,$n3z,$n3y> }\n";
  }
}


sub pov_voxel_ {
  if ( $in_voxels ) {
    if ($dimensions==2) {
      print "box { <$x,$y,0>, <$x2,$y2,$depth> }\n";
    } elsif ($dimensions==3) {
      my $enlarge = 1.001;
      my ($dx, $dy, $dz) = ( $enlarge*($x2-$x), $enlarge*($y2-$y), $enlarge*($z2-$z) );
      ($x2,$y2,$z2) = ($x+$dx, $y+$dy, $z+$dz);
      print "box { <$x,$y,$z>, <$x2,$y2,$z2> }\n";
    }
  }
}


sub pov_sphere_ {
  if ( $in_spheres ) {
    if ($dimensions==2) {
      print "cylinder { <$x,$y,0>, <$x,$y,$depth>, $radius }\n";
    } elsif ($dimensions==3) {
      print "sphere { <$x,$z,$y>, $radius }\n";
    }
  }
}
