/***************************************************************************
                          box.cpp  -  description
                             -------------------
    begin                : Sat Mar 1 2003
    copyright            : (C) 2003 by Moritz Franosch
    email                : mail@Franosch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "box.h"


using namespace std;


Box::Box(const Particle::vector_t& p1_ini, const Particle::vector_t& p2_ini) : 
  p1(p1_ini),
  p2(p2_ini) {
  REQUIRE(p1_ini.length()==p2_ini.length());
}

 
bool Box::operator()(const Particle::vector_t& p) const {
  REQUIRE(p1.length()==p.length());
  I(p1.length()==p2.length());
  for (int i=0; i<p.length(); ++i) {
    if ( ( p1[i]<=p[i] && p[i]<=p2[i] ) ||
	 ( p2[i]<=p[i] && p[i]<=p1[i] ) ) {
      // coordinate p[i] is between the corners
      // '<=' instead of '<' because in case of two dimensions, z=0
    } else {
      // coordinate p[i] is outside the corners
      // so p is outside
      return false;
    }
  }
  return true;
}


const Particle::vector_t& Box::get_corner1() const {
  return p1;
}


const Particle::vector_t& Box::get_corner2() const {
  return p2;
}


void Box::to_xml(std::ostream& o) {
  o << "<box>" << endl;
  o << "  <corner1> " 
    << Particle::list_vector(get_corner1()) << " </corner1>" << endl;
  o << "  <corner2> " 
    << Particle::list_vector(get_corner2()) << " </corner2>" << endl;
  o << "  </box>" << endl;
}
