/***************************************************************************
                          box.h  -  description
                             -------------------
    begin                : Sat Mar 1 2003
    copyright            : (C) 2003 by Moritz Franosch
    email                : mail@Franosch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BOX_H
#define BOX_H

#include "geometry.h"
#include "particle.h"


/** A box.
 *
 *  @author Moritz Franosch
 */
class Box : public Geometry {

private:

  Particle::vector_t p1, p2; // corners of the box

public: 

  /** Constructs a box with corners p1 and p2.
   */
  Box(const Particle::vector_t& p1, const Particle::vector_t& p2);

  /** True if position p is inside of the box.
   */
  virtual bool operator()(const Particle::vector_t& p) const;

  /** Returns a corner.
   */
  const Particle::vector_t& get_corner1() const;

  /** Returns the opposite corner.
   */
  const Particle::vector_t& get_corner2() const;

  /** Writes the box as XML into o.
   */
  virtual void to_xml(std::ostream& o);

};


#endif // BOX_H
