/***************************************************************************
                          compound.cpp  -  description
                             -------------------
    begin                : Sat Mar 1 2003
    copyright            : (C) 2003 by Moritz Franosch
    email                : mail@Franosch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "voxel.h"
#include "pyramid.h"
#include "compound.h"


using namespace std;


Compound::Compound() :
  objects(),
  objects_iterator(objects.begin()) {
}


void Compound::add(const Pointer<Geometry> g) {
  objects.push_back(g);
}


bool Compound::operator()(const Particle::vector_t& p) const {
  for (list< Pointer<Geometry> >::const_iterator i=objects.begin();
       i!=objects.end(); ++i) {
    const Geometry& geometry=**i;
    if ( geometry(p) ) {
      return true;
    }
  }
  return false;
}


void Compound::add_voxel(const Particle::vector_t& p1,
				    const Particle::vector_t& p2) {
  Pointer<Geometry> voxel(new Voxel(p1,p2) );
  add(voxel);
}


void Compound::add_pyramid(const Particle::vector_t& top,
			   const Particle::vector_t& c1,
			   const Particle::vector_t& c2,
			   const Particle::vector_t& c3,
			   const Particle::vector_t& c4) {
  Pointer<Geometry> pyramid(new Pyramid(top,c1,c2,c3,c4) );
  add(pyramid);
}


void Compound::reset() {
  objects_iterator = objects.begin();
}


bool Compound::next() {
  REQUIRE(!at_end());
  ++objects_iterator;
  return !at_end();
}


bool Compound::at_end() const {
  return objects_iterator==objects.end();
}


bool Compound::empty() const {
  return objects.empty();
}


int Compound::size() const {
  return objects.size();
}


const Geometry& Compound::get_object() const {
  return **objects_iterator;
}


Geometry& Compound::get_object() {
  return **objects_iterator;
}


void Compound::to_xml(std::ostream& o) {
  o << "<voxels>" << endl;
  // For each object ...
  for (reset(); !at_end(); next()) {
    // ... write it as XML into o.
    get_object().to_xml(o);
  }
  o << "</voxels>" << endl;
}
