/***************************************************************************
                          compound.h  -  description
                             -------------------
    begin                : Sat Mar 1 2003
    copyright            : (C) 2003 by Moritz Franosch
    email                : mail@Franosch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef COMPOUND_H
#define COMPOUND_H

#include <list>

#include "askosdata/pointer.h"

#include "geometry.h"


/** Compound geometric object.
 *
 *  @author Moritz Franosch
 */
class Compound : public Geometry {

private:

  std::list< Pointer<Geometry> > objects; // all geometric objects in this compound
  std::list< Pointer<Geometry> >::iterator objects_iterator; // iterator through objects

public: 

  /** Constructs an empty compound.
   */
  Compound();

  /** Adds geometric object g.
   */
  void add(const Pointer<Geometry> g);

  /** True if position p is inside any of the object that have been
   *  added.
   */
  virtual bool operator()(const Particle::vector_t& p) const;

  /** Adds a voxel with opposite corners p1 and p2.
   */
  void add_voxel(const Particle::vector_t& p1,
		 const Particle::vector_t& p2);

  /** Adds a pyramid with top and a base surface with edges c1,
   *  c2, c3 and c4.
   */
  void add_pyramid(const Particle::vector_t& top,
		   const Particle::vector_t& c1,
		   const Particle::vector_t& c2,
		   const Particle::vector_t& c3,
		   const Particle::vector_t& c4);

  /** Resets iterator through objects.
   */
  void reset();

  /** Iterator set to next object.
   *
   *  Returns false if no next object exists.
   */
  bool next();

  /** Last next() has returned false.
   */
  bool at_end() const;
  
  /** Returns true iff compound has no elements.
   */
  bool empty() const;

  int size() const;

  /** Returns a constant reference to the current object.
   */
  const Geometry& get_object() const;

  /** Returns a reference to the current object.
   */
  Geometry& get_object();

  /** Writes the compound as XML into o.
   */
  virtual void to_xml(std::ostream& o);

};


#endif // COMPOUND_H
