/***************************************************************************
                          cusp_kernel.h  -  description
                             -------------------
    begin                : Sat Jan 11 2003
    copyright            : (C) 2003 by Moritz Franosch
    email                : mail@Franosch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CUSP_KERNEL_H
#define CUSP_KERNEL_H

#include <kernel.h>


/** Implements the Cusp kernel W(r) = alpha_d * (1-r)^2 for r<1, 0
 *  else.
 *
 *  @author Moritz Franosch
 */
class Cusp_Kernel : public Kernel {

public: 

  /** Constructor, see class Kernel.
   */
  Cusp_Kernel(const int d, const double h);

  /** Returns w(x)=(1-x)^2 for x<1, 0 else.
   */
  virtual double w(const double x) const;

  /** Returns dw(x)/dx.
   */
  virtual double dw(const double x) const;

};


#endif // CUSP_KERNEL_H
