/***************************************************************************
                          kernel.cpp  -  description
                             -------------------
    begin                : Sat Jan 11 2003
    copyright            : (C) 2003 by Moritz Franosch
    email                : mail@Franosch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "mylimits.h"

#include <cmath>

#include "blitz/tinyvec-et.h"

#include "errorcheck.h"

#include "kernel.h"


using namespace std;
using namespace blitz;


Kernel::Kernel(const int d_ini, const double h_ini, const double alpha_d) :
  d(d_ini),
  h(h_ini),
  c_d(alpha_d/pow(h,d)) {
  REQUIRE(h>0);
  REQUIRE(d>0);
  ENSURE(c_d>0);
}


Kernel::~Kernel() {
}


double Kernel::get_h() const {
  return h;
}


int Kernel::get_d() const {
  return d;
}


double Kernel::dW(const double r) const {
  return c_d/h * dw(r/h);
}


const Particle::vector_t Kernel::grad_r(const Particle::vector_t& r1, 
					const Particle::vector_t& r2) const {
  REQUIRE(r1.length()==r2.length());
  // grad r1 W(|r1-r2|) = (r1-r2)/|r1-r2| * dW(|r1-r2|);
  /*
  double r12 = euclidian_distance(r1,r2);
  if ( r12 < numeric_limits<double>::epsilon() ) {
    return Particle::vector_t(0);
  }
  return dW(r12)/r12 * (r1-r2);
  */
  return grad_r(r1,r2,euclidian_distance(r1,r2));
}


const Particle::vector_t Kernel::grad_r(const Particle::vector_t& r1, 
					const Particle::vector_t& r2,
					const double& r12) const {
  REQUIRE( r1.length()==r2.length() );
  REQUIRE( r12==euclidian_distance(r1,r2) );
  // grad r1 W(|r1-r2|) = (r1-r2)/|r1-r2| * dW(|r1-r2|);
  if ( r12 < numeric_limits<double>::epsilon() ) {
    return Particle::vector_t(0);
  }
  return dW(r12)/r12 * (r1-r2);
}


const Particle::vector_t& Kernel::grad_r(const Particle::vector_t& r1, 
					 const Particle::vector_t& r2,
					 const double& r12,
					 Particle::vector_t& g) const {
  REQUIRE( r1.length()==r2.length() );
  /*
  if ( abs(r12-euclidian_distance(r1,r2)) >= 10*numeric_limits<double>::epsilon() ) {
    cerr << "r12=" << r12 << endl;
    cerr << "d=" << euclidian_distance(r1,r2) << endl;
    cerr << "r1=" << r1 << endl;
    cerr << "r2=" << r2 << endl;
    cerr << "r1-r2=" << r1-r2 << endl;
  }
  */
  REQUIRE( abs(r12-euclidian_distance(r1,r2)) < numeric_limits<double>::epsilon() );
  // grad r1 W(|r1-r2|) = (r1-r2)/|r1-r2| * dW(|r1-r2|);
  if ( r12 < numeric_limits<double>::epsilon() ) {
    g=0;
    return g;
  }
  //  g = dW(r12)/r12 * (r1-r2);
  const double c=dW(r12)/r12;
  for (int i=0; i<r1.length(); ++i) {
    g[i]=c*(r1[i]-r2[i]);
  }
  return g;
}


double select(const int i, double x1, double x2, double x3) {
  if (i==1) {
    return x1;
  }
  if (i==2) {
    return x2;
  }
  if (i==3) {
    return x3;
  }
  return numeric_limits<double>::quiet_NaN();
}
