/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Fri Jan  3 15:12:32 CET 2003
    copyright            : (C) 2003 by Moritz Franosch
    email                : mail@Franosch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <cstdlib>

#include "askosio/log.h"

#include "testvector.h"
#include "testkernel.h"
#include "testsph.h"
#include "testxml.h"
#include "testintegration.h"
#include "testlimits.h"
#include "rax_parser_sph.h"


using namespace std;


int main(int argc, char *argv[]) {
  cerr << "PartSim Version " << VERSION << "." << endl;
  
#if ( !defined(WITHOUT_NANA) || defined(BZ_DEBUG) )
  cerr << "Warning: This is a debugging version. Performance may be poor." << endl;
#endif

  LOG(log);
  //  log.get_default_logdevice().debug_on("Triangulator");

  if ( argc<=1 || ( string(argv[1])=="--test" && argc<=2 ) ) {
    cerr << "usage: " << argv[0] << " [ --test TEST ] [inputfile1] [inputfile2] ..." 
	 << endl;
    return EXIT_FAILURE;
  }

  if ( string(argv[1])=="--test" ) {
    string name=argv[2];

    if (name=="failure") {
      cerr << "The failure test fails." << endl;
      return EXIT_FAILURE;
    } else if (name=="success") {
      cerr << "The success test succeeds." << endl;
      return EXIT_SUCCESS;
    } else if (name=="vector") {
      TestVector test;
      return test();
    } else if (name=="kernel") {
      TestKernel test;
      return test();
    } else if (name=="sph") {
      TestSPH test;
      return test();
    } else if (name=="xml") {
      TestXML test;
      return test();
    } else if (name=="limits") {
      TestLimits test;
      return test();
    } else if (name=="integration") {
      TestIntegration test;
      return test();
    }

    cerr << "No such test '" << name << "'." << endl;
    return EXIT_FAILURE;
  }

  RAX_Parser_SPH parser;

  // parse files given as command line arguments
  for (int i=1; i<argc; ++i) {
    parser.parse(argv[i]);
  }

  return EXIT_SUCCESS;
}
