/***************************************************************************
                          smooth_triangle.h  -  description
                             -------------------
    begin                : Sun Nov 16 2003
    copyright            : (C) 2003 by Moritz Franosch
    email                : mail@Franosch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SMOOTH_TRIANGLE_H
#define SMOOTH_TRIANGLE_H

#include "triangle.h"


/** A smooth triangle (with normals on each corner).
 *
 *  @author Moritz Franosch
 */
class Smooth_Triangle : public Triangle {

private:

  Particle::vector_t n1, n2, n3; // normals of the triangle

public: 

  /** Constructs a triangle with corners p1, p2, p3 and
   *  normals n1, n2, n3.
   */
  Smooth_Triangle(const Particle::vector_t& p1, const Particle::vector_t& p2,
		  const Particle::vector_t& p3,
		  const Particle::vector_t& n1, const Particle::vector_t& n2,
		  const Particle::vector_t& n3);

  /** Returns the normal on the first corner.
   */
  const Particle::vector_t& get_normal1() const;

  /** Returns the normal on the second corner.
   */
  const Particle::vector_t& get_normal2() const;

  /** Returns the normal on the third corner.
   */
  const Particle::vector_t& get_normal3() const;

  /** Writes the triangle as XML into o.
   */
  virtual void to_xml(std::ostream& o);

};



#endif // SMOOTH_TRIANGLE_H
