/***************************************************************************
                          sph_space.cpp  -  description
                             -------------------
    begin                : Thu Jan 16 2003
    copyright            : (C) 2003 by Moritz Franosch
    email                : mail@Franosch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "askosio/log.h"

#include "sph_space.h"


using namespace std;


SPH_Space::SPH_Space(const double length_x, const double length_y, 
		     const double length_z, 
		     const double diameter) :
  Space(length_x,length_y,length_z,diameter),
  step(0) {
}


void SPH_Space::ensure_boundary_condition(Particle& particle) {
  LOG(log);

  double r_v=0.9; // fixme 0.9;

  Particle::vector_t p=particle.get_position();
  Particle::vector_t v=particle.get_velocity();

  // For each direction ...
  for (int i=0; i<Particle::vector_t::numElements; ++i) {

    // ... check if out of bounds ...
    bool out_of_bounds=false;

    // ... either because position is too small ...
    if ( p[i]<0 ) {
      p[i]=0;
      v *= r_v;
      v[i]=-v[i];
      out_of_bounds=true;
    }

    // ... or too high.
    if ( p[i]>get_length(i) ) {
      p[i]=get_length(i);
      v *= r_v;
      v[i]=-v[i];
      out_of_bounds=true;
    }

    // If out of bounds ...
    if (out_of_bounds) {

      // update positions ...
      particle.set_position(p);
      particle.set_velocity(v);

      // ... and warn user. fixme: generates MBs output
      // log.warn() << "Particle at position " << Particle::string_vector(p) 
      //	 << " was out of bounds." << endl;
    }
  }

  /*
  if ( p[0]<0 ) {
    p[0]=0;
    v *= r_v;
    v[0]=-v[0];
  }
  if ( p[0]>get_length_x() ) {
    p[0]=get_length_x();
    v *= r_v;
    v[0]=-v[0];
  }
  if ( p[1]<0 ) {
    p[1]=0;
    v *= r_v;
    v[1]=-v[1];
  }
  if ( p[1]>get_length_y() ) {
    p[1]=get_length_y();
    v *= r_v;
    v[1]=-v[1];
  }
  if ( p[2]<0 ) {
    p[2]=0;
    v *= r_v;
    v[2]=-v[2];
  }
  if ( p[2]>get_length_z() ) {
    p[2]=get_length_z();
    v *= r_v;
    v[2]=-v[2];
  }
  */
}


void SPH_Space::set_particle_mass(const double m) {
  get_particle().set_mass(m);
}


void SPH_Space::set_particle_density(const double rho) {
  get_particle().set_density(rho);
}


void SPH_Space::set_particle_drho(const double& drho) {
  get_particle().set_drho(drho);
}


void SPH_Space::set_particle_acceleration(const Particle::vector_t& a) {
  get_particle().set_acceleration(a);
}


double SPH_Space::update_particles(const double dt) {
  /*
  for ( reset(); !at_end(); next() ) {
    get_particle().update_Euler(dt);
    ensure_boundary_condition( get_particle() );
  }
  reorder_particles();
  return dt;
  */

  double delta_t=0;
  for ( reset(); !at_end(); next() ) {
    delta_t = get_particle().update_modified_midpoint(step,dt);
    ensure_boundary_condition( get_particle() );
  }
  step = (step+1) % 3;
  reorder_particles();
  return delta_t;
}
