/***************************************************************************
                          sphere_space.cpp  -  description
                             -------------------
    begin                : Sun Feb 2 2003
    copyright            : (C) 2003 by Moritz Franosch
    email                : mail@Franosch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <ostream>

#include "askosmath/askosrand.h"

#include "sphere_space.h"
#include "sphere.h"
#include "sph.h"
#include "particle.h"


using namespace std;


bool Sphere_Space::is_inside_sphere(const Particle::vector_t& p) {
  for ( reset(); !at_end(); next() ) {
    if ( sqr_euclidian_distance(p,get_sphere().get_position()) <=
	 sqr(get_sphere().get_radius()) ) {
      return true;
    }
  }
  return false;
}


Sphere& Sphere_Space::get_sphere() {
  ENSURE( !at_end() );
  return *sphere_iterator;
}


Sphere_Space::Sphere_Space(SPH& sph, const int nspheres) : 
  spheres(),
  sphere_iterator(spheres.begin()) {
  Uniform_Rand ux(0,sph.get_space().get_length_x());
  Uniform_Rand uy(0,sph.get_space().get_length_y());
  Uniform_Rand uz(0,sph.get_space().get_length_z());
  double R=max(max(sph.get_space().get_length_x(),sph.get_space().get_length_y()),
	       sph.get_space().get_length_z());
  bool created_sphere=false;
  while ( spheres.size() < nspheres || created_sphere ) {
    Particle::vector_t p;
    created_sphere=false;
    for (int i=0; i<1000; ++i) { // fixme

      // choose a random position
      I(p.length()==3);
      p[0]=ux();
      p[1]=uy();
      p[2]=uz();

      if ( sph.is_inner_point(p) ) {
	// Position is inside the fluid ...
	if ( !is_inside_sphere(p) ) {
	  // ... but not inside a sphere
	  // so create new sphere ...
	  if ( spheres.size() < nspheres ) {
	    // if the maximim number has not yet been reached.
	    spheres.push_back(Sphere(p,R));
	    created_sphere=true;
	    break; // start new for-loop
	  }
	}
      } else {
	// position is outside the fluid
	if ( is_inside_sphere(p) ) {
	  // but inside a sphere
	  // so adjust the radius of the sphere
	  double r=euclidian_distance(get_sphere().get_position(),p)-
	    numeric_limits<double>::epsilon();
	  // make sphere a bit smaller so that does not contain the position
	  get_sphere().set_radius(r);
	}
      }

    }
  }
}


void Sphere_Space::reset() {
  sphere_iterator=spheres.begin();
}


bool Sphere_Space::next() {
  ENSURE( !at_end() );
  ++sphere_iterator;
  return !at_end();
}


bool Sphere_Space::at_end() const {
  return sphere_iterator==spheres.end();
}


const Sphere& Sphere_Space::get_sphere() const {
  ENSURE( !at_end() );
  return *sphere_iterator;
}


void Sphere_Space::to_xml(ostream& o) {
  o << "<spheres>" << endl;
  // For each sphere ...
  for ( reset(); !at_end(); next() ) {
    // ... print position and radius.
    get_sphere().to_xml(o);
  }
  o << "</spheres>" << endl;
}


std::ostream& operator<<(std::ostream& o, Sphere_Space& sphere_space) {
  sphere_space.to_xml(o);
  return o;
}
