/***************************************************************************
                          sphere_space.h  -  description
                             -------------------
    begin                : Sun Feb 2 2003
    copyright            : (C) 2003 by Moritz Franosch
    email                : mail@Franosch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SPHERE_SPACE_H
#define SPHERE_SPACE_H

#include <ostream>
#include <list>

#include "particle.h"
#include "sphere.h"


class SPH;


/** Approximates a 3D SPH structure with spheres.
 *
 *  @author Moritz Franosch
 */
class Sphere_Space {

private:
  
  std::list<Sphere> spheres; // all spheres
  std::list<Sphere>::iterator sphere_iterator; // iterator through spheres

  /** True if position p is inside a sphere.
   */
  bool is_inside_sphere(const Particle::vector_t& p);

  /** Returns a reference to the current sphere.
   */
  Sphere& get_sphere();

public: 

  /** Constructs a sphere space from a sph space.
   *
   *  @parameter nspheres 
   *  Number of spheres to construct.
   */
  Sphere_Space(SPH& sph, const int nspheres);

  /** Resets iterator through spheres.
   */
  void reset();

  /** Iterator set to next sphere.
   *
   *  Returns false if no next sphere exists.
   */
  bool next();

  /** Last next() has returned false.
   */
  bool at_end() const;

  /** Returns a reference to the current sphere.
   */
  const Sphere& get_sphere() const;

  /** Output as XML into o.
   */
  void to_xml(std::ostream& o);

};


/** Output of sphere_space as XML.
 */
std::ostream& operator<<(std::ostream& o, Sphere_Space& sphere_space);


#endif // SPHERE_SPACE_H
