/***************************************************************************
                          state_equation.cpp  -  description
                             -------------------
    begin                : Sun Jan 12 2003
    copyright            : (C) 2003 by Moritz Franosch
    email                : mail@Franosch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "askosmath/askosmath.h"

#include "errorcheck.h"
#include "state_equation.h"


using namespace std;


State_Equation::State_Equation(const double velocity_of_sound, const double rho_0_ini) :
  rho_0(rho_0_ini),
  c(velocity_of_sound) {
}


double State_Equation::get_pressure_from_density(const double rho) const {
  return sqr(c)*(rho-rho_0);
}


double State_Equation::get_energy(const double m, const double rho) const {
  REQUIRE(rho>0);
  REQUIRE(rho_0>0);
  // E_p = - int p dV = - int c^2 ( m/V - rho_0 ) dV
  return m*sqr(c)*( log(rho/rho_0) + rho_0/rho - 1 );
}


void State_Equation::set_rho_0(const double rho_0_ini) {
  rho_0=rho_0_ini;
}


void State_Equation::set_velocity_of_sound(const double c_ini) {
  c=c_ini;
}


double State_Equation::get_rho_0() const {
  return rho_0;
}


double State_Equation::get_velocity_of_sound() const {
  return c;
}
