/***************************************************************************
                          state_equation.h  -  description
                             -------------------
    begin                : Sun Jan 12 2003
    copyright            : (C) 2003 by Moritz Franosch
    email                : mail@Franosch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef STATE_EQUATION_H
#define STATE_EQUATION_H


/** Equation of state.
 *
 *  @author Moritz Franosch
 */
class State_Equation {

private:

  double rho_0; // equilibrium density
  double c; // velocity of sound

public: 

  /** Constructor.
   */
  State_Equation(const double velocity_of_sound, const double rho_0);

  /** Get pressure from density.
   */
  double get_pressure_from_density(const double rho) const;

  /** Get pressure from mass m and density rho. 
   */
  double get_energy(const double m, const double rho) const;

  /** Set equilibrium density.
   */
  void set_rho_0(const double rho_0);

  /** Set velocity of sound.
   */
  void set_velocity_of_sound(const double c);

  /** Get equilibrium density.
   */
  double get_rho_0() const;

  /** Get velocity of sound.
   */
  double get_velocity_of_sound() const;

};


#endif // STATE_EQUATION_H
