/***************************************************************************
                          testsph.cpp  -  description
                             -------------------
    begin                : Sun Jan 12 2003
    copyright            : (C) 2003 by Moritz Franosch
    email                : mail@Franosch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>

#include "blitz/tinyvec-et.h"
#include "errorcheck.h"

#include "askosmath/askosrand.h"
#include "askosdates/timespec.h"
#include "askosdates/timeval.h"
#include "askosdates/convert_date.h"

#include "sph.h"
#include "testsph.h"


using namespace std;


int TestSPH::operator()() {

  /* benchmark
  int dimensions=2;
  double width=1;
  double height=1;
  double depth=0;
  double h=0.1;
  double velocity_of_sound=5;
  double dt=0.001;

  int nparticles=1000;
  double simtime=0.1;

  double radius=0.4;

  double frames_per_s=4;
  double speed=0.1;
  */

  /*
  int dimensions=2;
  double width=1;
  double height=1;
  double depth=0;
  double h=0.1;
  double velocity_of_sound=3;
  double dt=0.002;

  int nparticles=500;
  double simtime=10;

  double radius=0.4;

  double frames_per_s=4;
  double speed=0.1;
  */


  int dimensions=2;
  double width=1;
  double height=1;
  double depth=0;
  double h=0.2;
  double velocity_of_sound=10;
  double dt=0.001;

  int nparticles=3600;
  double simtime=0.003;

  double radius=0.3;

  double frames_per_s=1;
  double speed=0.1;


  Particle::vector_t middle(0);
  middle[0]=0.5;
  //  middle[1]=1.1*radius;
  middle[1]=0.5;
  middle[2]=0;

  SPH sph(dimensions,
	  width,height,depth,
	  h,
	  velocity_of_sound,
	  dt);

  Uniform_Rand ux(0,width);
  Uniform_Rand uy(0,height);

  // insert particles into a sphere
  /*
  for (int i=0; i<nparticles; ++i) {
    for (;;) {
      Particle::vector_t p;
      p[0]=ux();
      p[1]=uy();
      p[2]=0;
      if ( euclidian_norm(p-middle)<radius ) {
	sph.insert_particle(p);
	break;
      }
    }
  }
  */

  double nparticles_row=round(sqrt(double(nparticles)));
  double d=width/nparticles_row;
  for (int nx=0; nx<nparticles_row; ++nx) {
    for (int ny=0; ny<nparticles_row; ++ny) {
      Particle::vector_t p;
      p[0]=d/2+nx*d;
      p[1]=d/2+ny*d;
      p[2]=0;
      if ( euclidian_norm(p-middle)<radius ) {
	sph.insert_particle(p);
      }
    }
  }

  /*
  double nparticles_row=round(sqrt(double(nparticles)));
  double d=width/nparticles_row;
  for (int nx=0; nx<nparticles_row; ++nx) {
    for (int ny=0; ny<nparticles_row; ++ny) {
      Particle::vector_t p;
      p[0]=d/2+nx*d;
      p[1]=d/2+ny*d;
      p[2]=0;
      if ( p[1] < 0.5+0.125*cos(2*pi*p[0]) ) {
	sph.insert_particle(p);
      }
    }
  }
  */

  /*
  for (int i=0; i<nparticles; ++i) {
    for (;;) {
      Particle::vector_t p;
      p[0]=ux();
      p[1]=uy();
      p[2]=0;
      if ( p[1] < 0.5+0.125*cos(2*pi*p[0]) ) {
	sph.insert_particle(p);
	break;
      }
    }
  }
  */

  // all particles are inserted
  sph.start();

  timeval starttime;
  gettimeofday(&starttime,0);
  timeval display_time=starttime;

  // generate XML output
  cout << "<simulation>" << endl;
  while ( sph.get_time()<simtime ) {

    timeval now;
    gettimeofday(&now,0);
    double elapsed_time=substract_timeval(now,starttime);
    double diff_time=sph.get_time()/speed-elapsed_time;
    if (diff_time>0) {
      timespec sleeptime;
      double_to_timespec(diff_time,sleeptime);
      nanosleep(&sleeptime,0);
      gettimeofday(&now,0);
    }
    double elapsed_time_since_display=substract_timeval(now,display_time);
    if (elapsed_time_since_display>1.0/frames_per_s) {
      cout << sph; // output sph as XML
      /*
      cerr << "t=" << sph.get_time() << endl;
      display_time=now;
      cout << "kill g0.s0" << endl;
      cout << "g0.s0 symbol 1" << endl
	   << "g0.s0 symbol size 0.25" << endl
	   << "g0.s0 symbol color 4" << endl // blau
	   << "g0.s0 symbol pattern 1" << endl
	   << "g0.s0 symbol fill color 4" << endl
	   << "g0.s0 symbol fill pattern 1" << endl
	   << "g0.s0 linestyle 0" << endl;
      for ( sph.reset(); !sph.at_end(); sph.next() ) {
	Particle::vector_t position=sph.get_particle().get_position();
	cout << "g0.s0 point " << position[0] << ", " << position[1] << endl;
      }
      cout << "redraw" << endl;
      */
    }

    // go one timestep
    sph.update();

  }
  cout << "</simulation>" << endl;

  return EXIT_SUCCESS;
}
