/***************************************************************************
                          tetrahedron.h  -  description
                             -------------------
    begin                : Sun Mar 30 2003
    copyright            : (C) 2003 by Moritz Franosch
    email                : mail@Franosch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TETRAHEDRON_H
#define TETRAHEDRON_H

#include "geometry.h"


/** A tetrahedron.
 *
 *  @author Moritz Franosch
 */
class Tetrahedron : public Geometry {

private:

  Particle::vector_t p1, p2, p3, p4; // corners of the tetrahedron

public: 

  /** Constructs a tetrahedron with corners p1, p2, p3 and p4.
   */
  Tetrahedron(const Particle::vector_t& p1,
	      const Particle::vector_t& p2,
	      const Particle::vector_t& p3,
	      const Particle::vector_t& p4);

  /** Not implemented.
   */
  virtual bool operator()(const Particle::vector_t& p) const;

  /** Returns the first corner.
   */
  const Particle::vector_t& get_corner1() const;

  /** Returns the second corner.
   */
  const Particle::vector_t& get_corner2() const;

  /** Returns the third corner.
   */
  const Particle::vector_t& get_corner3() const;

  /** Returns the fourth corner.
   */
  const Particle::vector_t& get_corner4() const;

  /** Writes the tetrahedron as XML into o.
   */
  virtual void to_xml(std::ostream& o);

};


#endif // TETRAHEDRON_H
