/***************************************************************************
                          voxelizer.h  -  description
                             -------------------
    begin                : Sun Mar 9 2003
    copyright            : (C) 2003 by Moritz Franosch
    email                : mail@Franosch.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef VOXELIZER_H
#define VOXELIZER_H


#include <list>

#include "compound.h"
#include "voxel.h"


class SPH;


/** Vozelizes a sph space.
 *
 *  @author Moritz Franosch
 */
class Voxelizer {

private:

  Compound voxels; // all voxels that the voxelizer generates

  /** If the voxel Voxel(p1,p2) totally belongs to the sph space, then
   *  add it.
   *
   *  Else if the voxel only belongs partly to the sph space, then add
   *  smaller voxels recursively until the diameter is less than
   *  d_min.
   */
  void add_and_recurse(SPH& sph,
		       const Particle::vector_t& p1,
		       const Particle::vector_t& p2, 
		       const double d_min);

  /** Add subvoxels to approximate the surface.
   *
   *  The points p1, p2, p3, p4 are the corners of a rectangle in
   *  cyclic order.
   *
   *  The vector pn is normal to the rectangle and determines the area
   *  in which the surface is searched, i.e. the surface is searched
   *  on the lines [p1,p1+pn], [p2,p2+pn], ...
   */
  /*
  void add_subvoxel(SPH& sph, 
		    const Particle::vector_t& p1,
		    const Particle::vector_t& p2,
		    const Particle::vector_t& p3,
		    const Particle::vector_t& p4,
		    const Particle::vector_t& pn);
  */

  /** Returns the largest height on line [p0,p0+pn] such that point
   *  p0+height*pn/|pn| is still inside.
   */
  /*
  double get_largest_height(SPH& sph, 
			    const Particle::vector_t& p0,
			    const Particle::vector_t& pn);
  */

public:

  /** Constructs a voxelized space from a sph space.
   *
   *  The voxels start at diameter sph.get_d() and are recursively
   *  made smaller until d_min.
   *
   *  @parameter d_min
   *  Minimum diameter of voxels.
   */
  Voxelizer(SPH& sph, const double d_min);

  /** Resets iterator through voxels.
   */
  void reset();

  /** Iterator set to next voxel.
   *
   *  Returns false if no next voxel exists.
   */
  bool next();

  /** Last next() has returned false.
   */
  bool at_end() const;

  /** Returns a reference to the current voxel.
   */
  const Geometry& get_voxel() const;

  /** Output of voxelizer as XML.
   */
  void to_xml(std::ostream& o);

};


/** Output of voxelizer as XML.
 */
std::ostream& operator<<(std::ostream& o, Voxelizer& voxelizer);


#endif // VOXELIZER_H
